#include "forms.h"

GArray *observations_runs_list;
GArray *meteors_list;
GArray *charts_meteors_list;

void forms_init( int  *argc,
		 char ***argv )
{
  extern GArray *observations_runs_list;
  extern GArray *meteors_list;
  extern GArray *charts_meteors_list;
  
  corrida_printf ("initializing forms module...");

  /* set up list arrays */
  observations_runs_list = g_array_new (FALSE, TRUE, 
					sizeof (corrida_observation_run));
  meteors_list = g_array_new (FALSE, TRUE,
			      sizeof (corrida_meteor_record));
  charts_meteors_list = g_array_new (FALSE, TRUE,
				     sizeof (corrida_charts_meteor_record));

  /* FIXME: 3 calls below are probably temporary */
  print_data_from_runs_list ();
  print_data_from_meteors_list ();
  print_data_from_charts_meteors_list ();
}

void add_observation_run_record_to_garray( void )
{
  extern GArray *observations_runs_list;
  corrida_observation_run run_record;

  run_record.time_begin = g_strdup ("00:00");
  run_record.time_end = g_strdup ("00:00");
  run_record.teff = g_strdup ("1.00");
  run_record.lm = g_strdup ("0.00");
  run_record.f = g_strdup ("1.00");
  run_record.last_meteor = g_strdup ("0");

  g_array_append_val (observations_runs_list, run_record);
}

void remove_observation_run_record_from_garray( gint index )
{
  extern GArray *observations_runs_list;
  
  g_array_remove_index (observations_runs_list, index);
}

void clear_observations_runs_garray( void )
{
  extern GArray *observations_runs_list;
  gint len;
  gint it;

  len = observations_runs_list->len;

  for ( it = len-1; it >= 0; it-- ) {
    remove_observation_run_record_from_garray (it);
  }
}

void add_meteor_record_to_garray( void )
{
  extern GArray *meteors_list;
  corrida_meteor_record met_record;
  gint i = meteors_list->len - 1; /* i points on last member of
				   * meteors list
				   */
  gint next_meteor;
  char next_meteor_string[50];
  corrida_meteor_record last_record;

  if ( meteors_list->len > 0 ) { /* there are meteors in GArray */
    last_record = g_array_index (meteors_list, 
				 corrida_meteor_record,
				 i);
    next_meteor = atoi (last_record.no) + 1;
    sprintf (next_meteor_string, "%d", next_meteor);
  } else { /* GArray is empty */
    next_meteor = 1;
    sprintf (next_meteor_string, "%d", next_meteor);
  }

  met_record.no = g_strdup (next_meteor_string);
  met_record.time = g_strdup ("00:00");
  met_record.mag = g_strdup ("0.0");
  met_record.vel = g_strdup ("0");
  met_record.stream = g_strdup ("\0");
  
  g_array_append_val (meteors_list, met_record);
}

void remove_meteor_record_from_garray( gint index )
{
  extern GArray *meteors_list;
  
  g_array_remove_index (meteors_list, index);
}

void sort_corrida_meteors_in_garray( GArray *charts_meteors_list )
{
  g_array_sort (charts_meteors_list,
		compare_corrida_meteor_record);

}

gint compare_corrida_meteor_record( gconstpointer a,
				    gconstpointer b)
{
  gchar no1_str[20], no2_str[20];
  gint no1, no2;
  gint stat = 0;

  strcpy (no1_str, ((corrida_meteor_record *)a)->no);
  strcpy (no2_str, ((corrida_meteor_record *)b)->no);

  sscanf (no1_str, "%d", &no1);
  sscanf (no2_str, "%d", &no2);

  if ( no1 < no2 )
    stat = -1;
  else if ( no1 > no2 )
    stat = 1;

  return stat;
}

void clear_meteors_garray( void )
{
  extern GArray *meteors_list;
  gint len;
  gint it;

  len = meteors_list->len;

  for ( it = len-1; it >= 0; it-- ) {
    remove_meteor_record_from_garray (it);
  }
}

void add_charts_meteor_record_to_garray( corrida_charts_meteor_record charts_met_rec)
{
  extern GArray *charts_meteors_list;

  g_array_append_val (charts_meteors_list, charts_met_rec);
}

void remove_charts_meteor_from_garray( gint index )
{
  extern GArray *charts_meteors_list;

  g_array_remove_index (charts_meteors_list, index);
}

void sort_corrida_charts_meteors_in_garray( GArray *charts_meteors_list )
{
  g_array_sort (charts_meteors_list,
		compare_corrida_charts_meteor_record);
}

gint compare_corrida_charts_meteor_record( gconstpointer a,
					   gconstpointer b )
{
  gint no1, no2;
  gint stat = 0;

  no1 = ((corrida_charts_meteor_record *)a)->met_no;
  no2 = ((corrida_charts_meteor_record *)b)->met_no;
 
  if ( no1 < no2 )
    stat = -1;
  else if ( no1 > no2 )
    stat = 1;
  
  return stat;
}

void clear_charts_meteors_garray( void )
{
  extern GArray *charts_meteors_list;
  gint len;
  gint it;

  len = charts_meteors_list->len;

  for ( it = len-1; it >= 0; it-- ) {
    remove_charts_meteor_from_garray (it);
  }
}

/* check_string: checks various strings that can be inputed into lists.
 *               Return 0 if checked string is ok.
 *
 *               yessss... there are some unclean lines of code
 */
int check_string (gchar *string, int check_type)
{
  int check_stat = 0; /* check status: 0 - ok, -1 - error */
  gchar *original_string = string;
  double buf_double;
  gchar message[MAX_CORRIDA_STRLEN];
  gboolean dot_flag = FALSE; /* whether dot was counted */

  switch (check_type) {
  case CHECK_OBSERVER_CODE:
    if ( strlen (string) != 5 ) { /* length error */
      sprintf (message, "ERROR: wrong length of OBSERVER CODE (other than 5) string.");
      check_stat = -1;
    } else
      sprintf (message, "OBSERVER CODE format checked ok: %s", original_string);
    break;

  case CHECK_SITE_CODE:
    /* all site codes are treated as good */
    if ( 0 ) {
      ;
    } else
      sprintf (message, "SITE CODE format checked ok: %s", original_string);
    break;

  case CHECK_DATE:
    if ( strlen (string) != 11 ) { /* length error */
      sprintf (message, "ERROR: wrong length of DATE string (other than 11). You have typed '%s'", original_string);
      check_stat = -1;
    } else if ( !(isdigit (string[0]))  || /* format error */
		!(isdigit (string[1]))  || 
		!(isdigit (string[3]))  || 
		!(isdigit (string[4]))  || 
		!(isdigit (string[6]))  || 
		!(isdigit (string[7]))  || 
		!(isdigit (string[9]))  || 
		!(isdigit (string[10])) || 
		!(string[2] == '/')      ||
		!(string[5] == ' ')      ||
		!(string[8] == ' ')) {
      sprintf (message, "ERROR: wrong character encoutered in DATE string. You have typed '%s'", original_string);
      check_stat = -1;
    } else
      sprintf (message, "DATE format checked ok: %s", original_string);
    break;

  case CHECK_FIELD_ALPHA:
    if ( strlen (string) == 0 ) {
      sprintf (message, "ERROR: length of ALPHA (center of the observed field section) is 0", original_string);
      check_stat = -1;
    } else {
      while ( *string != '\0' ) {
	if ( !(isdigit (*string)) &&
	     !(isspace (*string)) &&
	     *string != '.' ) { /* wrong character encoutered */
	  sprintf (message, "ERROR: wrong character encoutered in ALPHA string (center of the observer field section). You have typed '%s'", original_string);
	  check_stat = -1;
	} else if ( *string == '.' ) {
	  if ( dot_flag == TRUE ) { /* ERROR: second dot encoutered */
	    sprintf (message, "ERROR: second dot encoutered in ALPHA string.(center of the observer field section). You have typed '%s', format is 'd.d'.", original_string);
	    check_stat = -1;
	    break;
	  } else { /* first count of dot */
	    dot_flag = TRUE;
	    string++;
	    continue;
	  }
	} else
	  string++;
      }
    }
    if ( check_stat == 0 )
      sprintf (message, "ALPHA format checked ok: %s", original_string);
    break;

  case CHECK_FIELD_DELTA:
    if ( strlen (string) == 0 ) {
      sprintf (message, "ERROR: length of DELTA (center of the observed field section) is 0", original_string);
      check_stat = -1;
    } else {
      while ( *string != '\0' ) {
	if ( !(isdigit (*string)) &&
	     !(isspace (*string)) &&
	     *string != '.' ) { /* wrong character encoutered */
	  sprintf (message, "ERROR: wrong character encoutered in DELTA string (center of the observer field section). You have typed '%s'", original_string);
	  check_stat = -1;
	} else if ( *string == '.' ) {
	  if ( dot_flag == TRUE ) { /* ERROR: second dot encoutered */
	    sprintf (message, "ERROR: second dot encoutered in DELTA string.(center of the observer field section). You have typed '%s', format is 'd.d'.", original_string);
	    check_stat = -1;
	    break;
	  } else { /* first count of dot */
	    dot_flag = TRUE;
	    string++;
	    continue;
	  }
	} else
	  string++;
      }
    }
    if ( check_stat == 0 )
      sprintf (message, "DELTA format checked ok: %s", original_string);
    break;

  case CHECK_TIME:
    if ( strlen (string) != 5 ) {
      sprintf (message, "ERROR: wrong length of TIME string. You have typed '%s', format is 'hh:mm'.", original_string);
      check_stat = -1;
    } else if (!isdigit (string[0]) || !isdigit (string[1]) ||
	     !isdigit (string[3]) || !isdigit (string[4]) ||
	     string[2] != ':') {
      sprintf (message, "ERROR: wrong character in TIME string. You have typed '%s', format is 'hh:mm'.", string);
      check_stat = -1;
    } else
      sprintf (message, "TIME format checked ok: %s", original_string);
    break;
    
  case CHECK_TEFF:
    while ( *string != '\0' ) {
      if ( isdigit (*string) ) { string++; continue; }
      else if ( *string == '.' ) {
	if ( dot_flag == TRUE ) { /* ERROR: second dot counted */
	  sprintf (message, "ERROR: second dot counted in TEFF string. You have typed '%s', format is 'd.dd'.", original_string);
	  check_stat = -1;
	  break;
	} else { /* first count of dot */
	  dot_flag = TRUE;
	  string++;
	  continue;
	}
      }
      else { /* format error */
	sprintf (message, "ERROR: wrong character in TEFF string. You have typed '%s', format is d.dd'.", original_string);
	check_stat = -1;
	break;
      }
    }
    if (check_stat == 0)
      sprintf (message, "TEFF format checked ok: %s", original_string);
    break;

    
  case CHECK_MAG:
     if (*string == '+' || *string == '-') string++; /* omit sign */
    
    while (*string != '\0') {
      if (isdigit (*string)) { string++; continue; }
      else if (*string == '.') {
	if (dot_flag == TRUE) { /* ERROR: second dot counted */
	  sprintf (message, "ERROR: second dot counted in MAGNITUDE string. You have typed '%s', format is '[sign]d.dd'.", original_string);
	  check_stat = -1;
	  break;
	} else { /* first count of dot */
	  dot_flag = TRUE;
	  string++;
	  continue;
	}
      }
      else { /* format error */
	sprintf (message, "ERROR: wrong character in MAGNITUDE string. You have typed '%s', format is '[sign]d.dd'.", original_string);
	check_stat = -1;
	break;
      }
    }
    if (check_stat == 0)
      sprintf (message, "MAGNITUDE format checked ok: %s", original_string);
    break;

  case CHECK_F:
    while (*string != '\0') {
      if (isdigit (*string)) { string++; continue; }
      else if (*string == '.') {
	if (dot_flag == TRUE) { /* ERROR: second dot counted */
	  sprintf (message, "ERROR: second dot counted in F factor string. You have typed '%s', format is 'd.dd'.", original_string);
	  check_stat = -1;
	  break;
	} else { /* first count of dot */
	  dot_flag = TRUE;
	  string++;
	  continue;
	}
      }
      else { /* format error */
	sprintf (message, "ERROR: wrong character in F factor string. You have typed '%s', format is d.dd'.", original_string);
	check_stat = -1;
	break;
      }
    }
    if (check_stat == 0) {
      buf_double = atof (original_string);
      if (buf_double>= 1) /* f factor must be >= 1 */
	sprintf (message, "F format checked ok: %s", original_string);
      else { /* ERROR: f factor < 1 */
	sprintf (message, "ERROR: F factor < 1. You have typed '%s', format is 'd.dd' and F must be > 1.", original_string);
	check_stat = -1;
	break;
      }
    }
    break;

  case CHECK_UINT:
    while (*string != '\0') {
      if (!isdigit (*string)) {
	sprintf (message, "ERROR: not digit character encoutered in UNSIGNED INTEGER string. You have typed '%s'", original_string);
	check_stat = -1;
	break;
      } else
	string++;
    }
    if (check_stat == 0) {
      sprintf (message, "UNSIGNED INTEGER format checked ok: %s", original_string);
      break;
    }
    break;

  case CHECK_FLOAT:
    if (*string == '-' || *string == '+') /* omit sign */
      string++;
    while (*string != '\0') {
      if ( !isdigit(*string) && !(*string == '.')) {
	sprintf (message, "ERROR: bad character encouterd in FLOAT string, You have typed '%s", original_string);
	check_stat = -1;
	break;
      } else
	string++;
    }
    if (check_stat == 0) {
      sprintf (message, "FLOAT format checked ok: %s", original_string);
      break;
    }
    break;

  case CHECK_VEL:
    while ( *string != '\0' ) {
      if ( isdigit (*string) ) { string++; continue; }
      else if (*string == '.') {
	if ( dot_flag == TRUE ) { /* ERROR: second dot encouterd */
	  sprintf (message, "ERROR: second dot encoutered in VELOCITY string. You have typed '%s', format is 'd.d'.", original_string);
	  check_stat = -1;
	  break;
	} else { /* first count of dot */
	  dot_flag = TRUE;
	  string++;
	  continue;
	}
      }
      else { /* format error */
	sprintf (message, "ERROR: wrong character in VELOCITY string. You have typed '%s', format is d.d'.", original_string);
	check_stat = -1;
	break;
      }
    }
    if ( check_stat == 0 )
      sprintf (message, "VELOCITY format checked ok: %s", original_string);
    break;

  case CHECK_COLOR:
    while ( *string != '\0' ) {
      if ( isdigit (*string) ) {
	sprintf (message, "ERROR: digit character encouterd in COLOR string. You have typed '%s'", original_string);
	check_stat = -1;
	break;
      }	else if (!isalpha (*string)) {
	sprintf (message, "ERROR: not alpha character encouterd in COLOR string. You have typed '%s'", original_string);
	check_stat = -1;
	break;
      } else
	string++;
    }
    if (check_stat == 0) {
      sprintf (message, "COLOR format checked ok: %s", original_string);
      break;
    }
    break;

  case CHECK_STREAM:
    while ( *string != '\0' ) {
      if ( isdigit (*string) ) {
	sprintf (message, "ERROR: digit character encouterd in STREAM string. You have typed '%s'", original_string);
	check_stat = -1;
	break;
      }	else if ( !isalpha (*string) ) {
	sprintf (message, "ERROR: not alpha character encouterd in STREAM string. You have typed '%s'", original_string);
	check_stat = -1;
	break;
      } else
	string++;
    }
    if ( check_stat == 0 ) {
      sprintf (message, "STREAM format checked ok: %s", original_string);
      break;      
    }
    break;
  }

  corrida_printf (message);

  return check_stat;
}

corrida_export_data create_export_data( void )
{
  corrida_export_data export_data;

  static GString *heads = NULL;
  static GString *coords = NULL;
  static GString *notpl = NULL;
  static gchar *observer_code = NULL;
  static gchar *site_code = NULL;

  extern GtkWidget *main_window;
  extern GtkWidget *observer_code_entry;
  extern GtkWidget *site_code_entry;
  extern GtkWidget *field_alpha_entry;
  extern GtkWidget *field_delta_entry;

  extern GtkWidget *date_entry;

  gboolean stat = TRUE;
  gchar date[MAX_CORRIDA_STRLEN];
  gchar f_alpha[MAX_CORRIDA_STRLEN];
  gchar f_delta[MAX_CORRIDA_STRLEN];

  /* initialize static vars */
  if ( heads != NULL )
    g_string_free (heads, TRUE);
  if ( coords != NULL )
    g_string_free (coords, TRUE);
  if ( notpl != NULL )
    g_string_free (notpl, TRUE);
  if ( observer_code != NULL )
    g_free (observer_code);
  if ( site_code != NULL )
    g_free (site_code);

  heads = g_string_new ("\0");
  coords = g_string_new ("\0");
  notpl = g_string_new ("\0");

  export_data.heads = heads;
  export_data.coords = coords;
  export_data.notpl = notpl;

  /* get & check observer code */
  if ( stat == TRUE ) { 
    observer_code = g_strdup (gtk_entry_get_text (GTK_ENTRY (observer_code_entry)));
    if ( check_string (observer_code, CHECK_OBSERVER_CODE) != 0 ) {
      corrida_error_box (GTK_WINDOW (main_window), "Please check OBSERVER CODE entry (observation frame).\nMore info on statusbar.");
      stat = FALSE; /* do not continue GArray processing */
    } else {
      export_data.observer_code = observer_code;
    }
  }

  /* get & check site code */
  if ( stat == TRUE ) {
    site_code = g_strdup (gtk_entry_get_text (GTK_ENTRY (site_code_entry)));
    if ( check_string (site_code, CHECK_SITE_CODE) != 0 ) {
      corrida_error_box (GTK_WINDOW (main_window), "Please check SITE CODE entry (observation frame).\nMore info on statusbar.");
      stat = FALSE; /* do not continue processing */
    } else {
      export_data.site_code = site_code;
    }
  }

  /* get & check date */
  if ( stat == TRUE ) {
    strcpy (date, gtk_entry_get_text (GTK_ENTRY (date_entry)));
    if ( check_string (date, CHECK_DATE) != 0 ) {
      corrida_error_box (GTK_WINDOW (main_window), "Please check DATE entry (observation frame).\nMore info on statusbar.");
      stat = FALSE; /* do not continue processing */
    }
  }

  /* get & check field alpha */
  if ( stat == TRUE ) {
    strcpy (f_alpha, gtk_entry_get_text (GTK_ENTRY (field_alpha_entry)));
    if ( check_string (f_alpha, CHECK_FIELD_ALPHA) != 0 ) {
      corrida_error_box (GTK_WINDOW (main_window), "Please check ALFA entry in the 'center observed field' section.\nMore info on statusbar.");
      stat = FALSE; /* do not continue processing */
    }
  }

  /* get & check field delta */
  if ( stat == TRUE ) {
    strcpy (f_delta, gtk_entry_get_text (GTK_ENTRY (field_delta_entry)));
    if ( check_string (f_delta, CHECK_FIELD_DELTA) != 0 ) {
      corrida_error_box (GTK_WINDOW (main_window), "Please check DELTA entry of the 'centero observed field' section.\nMore info on statusbar.");
      stat = FALSE; /* do not continue processing */
    }
  }

  /* process GArrays */
  if ( stat == TRUE ) {
    extern GArray *observations_runs_list;

    extern GArray *meteors_list;
    extern GArray *charts_meteors_list;   
    GArray *copy_meteors_list;
    GArray *copy_charts_meteors_list;

    corrida_observation_run garray_run_rec;
    corrida_meteor_record garray_met_rec;	  
    corrida_charts_meteor_record garray_ch_met_rec;

    gchar run_record[MAX_CORRIDA_STRLEN];
    gchar meteor_record[MAX_CORRIDA_STRLEN];
    gchar charts_meteor_record[MAX_CORRIDA_STRLEN];

    gint idx_to_processed_meteor = 0;
    gint num_of_processed_meteor = 0;
    gint last_meteor_in_previous_run = -1;

    gint run_it;
    gint day, dumb, month, year;
    gchar location[MAX_CORRIDA_STRLEN];
    gint time_beg_h, time_beg_m;
    gint time_end_h, time_end_m;
    double solar_longitude;
    double cen_alpha, cen_delta;
    double teff, f, lm;

    gint velocity_estimation_type;
    
    /* we will have to work on copys of meteors garrays,
     * so we create copys here
     */
    {
      gint it;

      copy_meteors_list = g_array_new (FALSE, FALSE,
				       sizeof (corrida_meteor_record));
      copy_charts_meteors_list = g_array_new (FALSE, FALSE,
					      sizeof (corrida_charts_meteor_record));
      /* copy meteors garrays */
      for ( it = 0; it < meteors_list->len; it++ ) {
	garray_met_rec = g_array_index (meteors_list,
					corrida_meteor_record,
					it);
	g_array_append_val (copy_meteors_list,
			    garray_met_rec);

      }
      for ( it = 0; it < charts_meteors_list->len; it++ ) {
	garray_ch_met_rec = g_array_index (charts_meteors_list,
					   corrida_charts_meteor_record,
					   it);	
	g_array_append_val (copy_charts_meteors_list,
			    garray_ch_met_rec);
			    
      }
      
      /* sort meteors garrays (copys of course) */
      sort_corrida_meteors_in_garray (copy_meteors_list);
      sort_corrida_charts_meteors_in_garray (copy_charts_meteors_list);
    }

    /* determine velocity estimation type for processed observation */
    velocity_estimation_type = get_velocity_estimation_type (copy_meteors_list);

    /* process runs array */
    for ( run_it = 0; run_it < observations_runs_list->len; run_it++ ) {
      sscanf (date, "%d/%d %d %d", &day, &dumb, &month, &year);
      
      export_data.date_day = day;
      export_data.date_month = month;
      export_data.date_year = year;

      get_location (site_code, location);
      
      garray_run_rec = g_array_index (observations_runs_list,
				      corrida_observation_run,
				      run_it);
      sscanf (garray_run_rec.time_begin, "%d:%d", &time_beg_h, &time_beg_m);
      sscanf (garray_run_rec.time_end, "%d:%d", &time_end_h, &time_end_m);
      
      solar_longitude = get_solar_longitude (date, 
					     time_beg_h, time_beg_m,
					     time_end_h, time_end_m);

      cen_alpha = get_center_alpha (f_alpha);
      cen_delta = get_center_delta (f_delta);

      teff = atof (garray_run_rec.teff);
      f = atof (garray_run_rec.f);
      lm = atof (garray_run_rec.lm);

      /*                   |ids    |co|     date     | t beg  | t end  |slong|  center|  |teff |  f  | lm  |*/
      sprintf (run_record, "%03d %s %s %02d %02d %02d %02d%02d %02d%02d %7.3f %3.0f %3.0f %4.2f %4.2f %4.2f %s\n",
	       run_it, observer_code, location,
	       day, month, year,
	       time_beg_h, time_beg_m,
	       time_end_h, time_end_m,
	       solar_longitude,
	       cen_alpha, cen_delta,
	       teff, f, lm, site_code); /* MEN WORKING - site code */

      /* append record to heads */
      g_string_append (heads, run_record);
      
      /* process meteors garrays */
      {
	gint last_met;
	gint it;

	sscanf (garray_run_rec.last_meteor, "%d", &last_met);

	while ( 1 ) {
	  if ( idx_to_processed_meteor < copy_meteors_list->len 
	       && last_meteor_in_previous_run < last_met) {
	    garray_met_rec = g_array_index (copy_meteors_list,
					    corrida_meteor_record,
					    idx_to_processed_meteor);
	  } else
	    break;
	
	  sscanf (garray_met_rec.no, "%d", &num_of_processed_meteor);

	  if ( num_of_processed_meteor > last_met ) {
	    last_meteor_in_previous_run = last_met;
	    break;
	  } else if ( num_of_processed_meteor <= last_met ) {
	    gchar date_string[MAX_CORRIDA_STRLEN];
	    gchar *time;
	    gdouble mag;
	    gdouble vel;
	    gint vel_degs;

	    corrida_charts_meteor_record *charts_meteor;

	    time = garray_met_rec.time;
	    mag = atof (garray_met_rec.mag);
	    vel = atof (garray_met_rec.vel);
	    vel_degs = vel_rel2vel_deg (vel, velocity_estimation_type);
	    create_date_string (date, date_string);

	    charts_meteor = get_meteor_from_charts_garray (charts_meteors_list,
							   num_of_processed_meteor);

	    if ( charts_meteor != NULL ) { /* print record to plotteds */
	      gdouble rac_beg, dec_beg;
	      gdouble rac_end, dec_end;

	      rac_beg = charts_meteor->equ_begin.alpha;
	      dec_beg = charts_meteor->equ_begin.delta;
	      rac_end = charts_meteor->equ_end.alpha;
	      dec_end = charts_meteor->equ_end.delta;
	      
	      sprintf (meteor_record, 
		       "%s %03d %5.1f %2d %s %6.2f %6.2f %6.2f %6.2f %s %03d\n",
		       date_string, num_of_processed_meteor, 
		       mag, vel_degs, time,
		       rac_beg, dec_beg,
		       rac_end, dec_end,
		       observer_code, run_it);
	      g_string_append (coords, meteor_record);

	    } else { /* print record to not plotteds */
	      gchar *stream;
	    
	      stream = garray_met_rec.stream;

	      sprintf (charts_meteor_record,
		       "%03d %s %s %s %03d %5.1f %2d %s\n",
		       run_it, observer_code, date_string,
		       time, num_of_processed_meteor, mag, vel_degs, stream);
	      g_string_append (notpl, charts_meteor_record);
	    }

	    idx_to_processed_meteor++;
	  }
	}
      }
    }
  }

  if ( stat == TRUE ) {
    /* convert strings to upper case */
    string_to_upper_case (export_data.heads->str);
    string_to_upper_case (export_data.coords->str);
    string_to_upper_case (export_data.notpl->str);
    string_to_upper_case (export_data.observer_code);
    string_to_upper_case (export_data.site_code);
  }

  export_data.stat = stat;

  return export_data;
}

void print_data_from_runs_list( void )
{
  if (PRINT_DATA_FROM_RUNS_LIST) {
    extern GArray *observations_runs_list;
    gint iter;
    corrida_observation_run run;

    printf ("=========================================\n");
    printf ("Data from 'GArray observations_runs_list':\n");
    
    for (iter = 0; iter < observations_runs_list->len; iter++) {
      run = g_array_index (observations_runs_list, corrida_observation_run, iter);
      printf ("%s %s %s %s %s %s\n", run.time_begin, run.time_end,
	      run.teff, run.lm, run.f, run.last_meteor);
    }
    printf ("=========================================\n");
  }
}

void print_data_from_meteors_list( void )
{
  if (PRINT_DATA_FROM_METEORS_LIST) {
    extern GArray *meteors_list;
    gint iter;
    corrida_meteor_record met_record;
    
    printf ("================================\n");
    printf ("Data from 'GArray meteors_list':\n");
    
    for ( iter = 0; iter < meteors_list->len; iter++ ) {
      met_record = g_array_index (meteors_list, corrida_meteor_record, iter);
      printf ("%s %s %s %s %s \n", met_record.no, met_record.time,
	      met_record.mag, met_record.vel, met_record.stream);
    }
    printf ("================================\n");
  }
}

void print_data_from_charts_meteors_list( void )
{
  if (PRINT_DATA_FROM_CHARTS_METEORS_LIST) {
    extern GArray *charts_meteors_list;
    gint iter;
    corrida_charts_meteor_record charts_met_record;

    printf ("=======================================\n");
    printf ("Data from 'GArray charts_meteors_list':\n");
    printf ("       cartesian coords                equatorial coords\n");

    for ( iter = 0; iter < charts_meteors_list->len; iter++ ) {
      charts_met_record = g_array_index (charts_meteors_list, 
					 corrida_charts_meteor_record,
					 iter);
      printf ("%d %d ", charts_met_record.met_no, charts_met_record.map_no);
      printf ("|| ");
      printf ("%6.2f %6.2f ", 
	      charts_met_record.car_begin.x, charts_met_record.car_begin.y);
      printf ("%6.2f %6.2f ",
	      charts_met_record.car_end.x, charts_met_record.car_end.y);
      printf ("|| ");
      printf ("%6.2f %6.2f ",
	      charts_met_record.equ_begin.alpha, charts_met_record.equ_begin.delta);
      printf ("%6.2f %6.2f ",
	      charts_met_record.equ_end.alpha, charts_met_record.equ_end.delta);
      printf ("\n");
    }
    printf ("=======================================\n");
  }
}

void get_location( gchar *site_code, gchar *location )
{
  static GArray *locations_base = NULL;
  corrida_location_record location_record;
  
  gchar location_string[MAX_CORRIDA_STRLEN];
  gchar *vmdbsite_table_path;
  FILE *sites_file;

  gchar site_id[10];
  gchar site_name[30];
  gchar site_country[30];

  gint lmd_deg, lmd_min;
  gchar lmd_sec_str[10];
  float lmd_sec;
  gchar lmd_dir[10];
  gint phi_deg, phi_min;
  gchar phi_sec_str[10];
  float phi_sec;
  gchar phi_dir[10];

  gint it;
  gboolean location_found = FALSE;
  gchar message[MAX_CORRIDA_STRLEN];

  vmdbsite_table_path = internal_data_get_vmdbsite_table_path ();

  strcpy (location_string, site_code); /* if we won't find record in database
					* we return site_code number we recived 
					*/

  /* initalize locations_base */
  if ( locations_base == NULL ) {
    locations_base = g_array_new (FALSE, FALSE,
				  sizeof (corrida_location_record));
  }

  /* try to find location in locations_base */
  for ( it = 0; it < locations_base->len; it++ ) {

      location_record = g_array_index (locations_base,
				       corrida_location_record,
				       it);

      if ( strcmp (location_record.site_code, site_code) == 0) { /* location found */
      sprintf (location_string, "%05.1f %c %5.1f %c", 
	       location_record.lmd, location_record.lmd_dir,
	       location_record.phi, location_record.phi_dir);

      location_found = TRUE;
      break;
    }
  }

  /* find location in file if it was not found  
   * locations base and add to locations base 
   */
  if ( location_found == FALSE ) {
    sites_file = fopen (vmdbsite_table_path, "r");
    while ( !(feof (sites_file)) ) {
      gchar line[MAX_VMDBSITE_STRLEN];
      
      fgetl (sites_file, line, MAX_VMDBSITE_STRLEN-1);
      sscanf (line, "%5s", site_id);

      if ( strcmp (site_code, site_id) == 0 ) { /* site found in file */
	strncpy (site_name, line+6, 20);
	site_name[20] = '\0';
	strncpy (site_country, line+26, 20);
	site_country[20] = '\0';
	sscanf (line+46, "%d %d %s %s %d %d %s %s",
		&lmd_deg, &lmd_min, lmd_sec_str, lmd_dir,
		&phi_deg, &phi_min, phi_sec_str, phi_dir);
	
	lmd_sec = atof (lmd_sec_str);
	phi_sec = atof (phi_sec_str);
	
	location_record.site_code = g_strdup (site_id);
	location_record.lmd = lmd_deg + lmd_min / 60.0 + lmd_sec / 3600.0;
	location_record.lmd_dir = ((char *)(strchr (lmd_dir, 'E')) != NULL) ? 'E' : 'W' ;
	location_record.phi = phi_deg + phi_min / 60.0 + phi_sec / 3600.0;
	location_record.phi_dir = ((char *)(strchr (phi_dir, 'N')) != NULL) ? 'N' : 'S';
	sprintf (location_string, "%05.1f %c %5.1f %c", 
		 location_record.lmd, location_record.lmd_dir,
		 location_record.phi, location_record.phi_dir); 

	g_array_append_val (locations_base, location_record);

	location_found = TRUE;

	break;
      }
    }
  fclose (sites_file); 
  }

  if ( location_found == FALSE ) {
    sprintf (message, "WARNING: there is no %s code in data base", site_code);
    corrida_printf (message);
  }

  strcpy (location, location_string);
}

gdouble get_solar_longitude( gchar *input_date,
			     gint time_beg_h, gint time_beg_m,
			     gint time_end_h, gint time_end_m)
{
  gchar date[MAX_CORRIDA_STRLEN];
  gdouble solar_longitude;
  gdouble time_beg, time_end, time;
  gchar *s_rec1;
  gchar *s_rec2;

  strcpy (date, input_date); /* we should work on copy of date */

  /* compute time for which we try find solar longitude */
  {
    time_beg = time_beg_h + time_beg_m / 60.0;
    time_end = time_end_h + time_end_m / 60.0;

    /* get next date if this is time after midnight */
    if ( (time_beg < 12.0) && (time_end < 12.0) ) {
      gchar date_buf[MAX_CORRIDA_STRLEN];
      get_next_date (date, date_buf);
      strcpy (date, date_buf);
    }
    
    if ( time_end < time_beg ) {
      time_end = time_end + 24.0;
    }

    time = (time_beg + time_end) / 2.0;

    if ( time > 24.0 ) {
      time = time - 24.0;
    }
  }

  if ( time < 22.0 ) { /* we need only one record */
    gint idx;
    gdouble sol_1, sol_2;
    gdouble buf;
    gdouble dumb;
    
    s_rec1 = get_solar_long_record (date);

    if ( s_rec1 == NULL ) /* we can not find solar long */ 
      {
	solar_longitude_not_found_message (date);
	
	solar_longitude = 999.999;
      } 
    else
      {
	idx = floor (time / 2.0);

	sol_1 = get_sol_from_record (s_rec1, idx);
	sol_2 = get_sol_from_record (s_rec1, idx+1);

	if ( sol_2 < sol_1 )
	  sol_2 = sol_2 + 360.0;

	buf = modf (time / 2.0, &dumb);

	solar_longitude = sol_1 + ( buf * (sol_2 - sol_1) );

	if ( solar_longitude > 360.0 )
	  solar_longitude = solar_longitude - 360.0;
      }
  }
  else { /* we need two records */
    gdouble sol_1, sol_2;
    gdouble buf;
    gdouble dumb;
      
    s_rec1 = get_solar_long_record (date);
    s_rec2 = get_next_solar_long_record (date);

    if ( (s_rec1 == NULL) || (s_rec2 == NULL) ) /* we can not find solar long */ 
      {
	solar_longitude_not_found_message (date);

	solar_longitude = 999.999;
      } 
    else 
      {
	sol_1 = get_sol_from_record (s_rec1, 11);
	sol_2 = get_sol_from_record (s_rec2, 0);

	if ( sol_2 < sol_1 )
	  sol_2 = sol_2 + 360.0;
	
	buf = modf (time / 2.0, &dumb);
	
	solar_longitude = sol_1 + ( buf * (sol_2 - sol_1) );
	
	if ( solar_longitude > 360.0 )
	  solar_longitude = solar_longitude - 360.0;
      }
  }

  return solar_longitude;
}

gchar *get_solar_long_record( gchar *date )
{
  static GArray *internal_solar_long_table = NULL;
  gchar *s_long_record;

  if ( internal_solar_long_table == NULL ) /* initialize internal table */
    internal_solar_long_table = g_array_new (FALSE, FALSE, sizeof (gchar *));

  s_long_record = get_solar_long_record_from_internal_table (internal_solar_long_table, date);

  if ( s_long_record == NULL ) /* record was not found in internal table */
    { 
      s_long_record = get_solar_long_record_from_file (date);
      
      if ( s_long_record != NULL ) /* add record to internal table */ 
	{
	  g_array_append_val (internal_solar_long_table, s_long_record);
	}
    }

  return s_long_record; /* record was not found in file */
}

gchar *get_solar_long_record_from_internal_table( GArray *internal_solar_long_table,
						  gchar *date )
{
  gint it;
  gchar *sol_long_record;
  gint day, month, year;
  gint seeking_day, seeking_month, seeking_year;

  sscanf (date, "%d/%*d %d %d", &seeking_day, &seeking_month, &seeking_year);  

  for ( it = 0; it < internal_solar_long_table->len; it++ ) {
    sol_long_record = g_array_index (internal_solar_long_table,
				     gchar *,
				     it);
    sscanf (sol_long_record, "%d/%d/%d", &day, &month, &year);

    if ( (day == seeking_day) 
	 && (month == seeking_month) 
	 && (year == seeking_year) ) {
      return sol_long_record;
    }
  }

  return NULL; /* record was not found in internal table */
}

gchar *get_solar_long_record_from_file( gchar *date )
{
  static gchar *solar_long_table_path = NULL;
  gchar solar_long_record[MAX_CORRIDA_STRLEN];
  gint day, month, year;
  gint seeking_day, seeking_month, seeking_year;
  FILE *table_fp;

  sscanf (date, "%d/%*d %d %d", &seeking_day, &seeking_month, &seeking_year);

  if ( solar_long_table_path == NULL ) /* initialize path to table */
    solar_long_table_path = internal_data_get_solar_long_table_path ();

  table_fp = fopen (solar_long_table_path, "r");

  while ( !(feof (table_fp)) ) {
    if ( !(feof (table_fp)) ) {
      fgetl (table_fp, solar_long_record, MAX_CORRIDA_STRLEN-1);
      sscanf (solar_long_record, "%d/%d/%d\n", &day, &month, &year);

      if ( (day == seeking_day) 
	   && (month == seeking_month) 
	   && (year == seeking_year) ) { /* we've found our seeking record */

	gchar *sol_long_p;
	gint num;

	num = strlen (solar_long_record) + 1;
	
	sol_long_p = (gchar *) calloc (num+1, sizeof (gchar));
	strcpy (sol_long_p, solar_long_record);

	return sol_long_p;
      }
    }
  }

  if ( table_fp != NULL )
    fclose (table_fp);
  
  return NULL;
}

gdouble get_sol_from_record (gchar *s_rec, gint idx)
{
  gint it;
  float sol_long;
  gchar specification[MAX_CORRIDA_STRLEN] = "%*d/%*d/%*d";

  for ( it = 0; it < idx; it++ )
    strcat (specification, "%*f");

  strcat (specification, "%f");

  sscanf (s_rec, specification, &sol_long);

  return sol_long;
}

gchar *get_next_solar_long_record( gchar *date )
{
  gchar next_date[MAX_CORRIDA_STRLEN];
  
  get_next_date (date, next_date) ;

  return get_solar_long_record (next_date);
}

void get_next_date( gchar *previous_date, gchar *next_date )
{
  gint leap;
  gint day, month, year;
  static gchar daytab[2][13] = {
    {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
    {0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
  };
  
  sscanf (previous_date, "%d/%*d %d %d", &day, &month, &year);  

  leap = year%4 == 0 && year%100 !=0 || year%400 == 0;

  day++;
  
  if ( day > daytab[leap][month] ) { /* next month */
    day = 1;
    month++;
    if ( month > 12 ) { /* next year */
      month = 1;
      year++;
    }
  }

  sprintf (next_date, "%02d/%02d %02d %02d", day, day+1, month, year);
}

void solar_longitude_not_found_message( gchar *date )
{
  gchar message[MAX_CORRIDA_STRLEN];

  sprintf (message, "WARNING: unable to find solar longitude for date: %s", date);

  corrida_printf (message);
}

double get_center_alpha( gchar *f_alpha )
{
  double center_alpha;

  int hours;
  float mins;

  sscanf (f_alpha, "%d %f", &hours, &mins);
  
  center_alpha = (360.0 * (hours + mins / 60.0) / 24.0) ;

  return center_alpha;
}

double get_center_delta( gchar *f_delta )
{
  double center_delta;
  
  int degs;
  float mins;

  sscanf (f_delta, "%d %f", &degs, &mins);

  center_delta =  degs + mins / 60.0;

  return center_delta;
}

corrida_charts_meteor_record *get_meteor_from_charts_garray( GArray *charts_meteors_list,
							     gint met_num )
{
  gint it;

  for ( it = 0; it < charts_meteors_list->len; it++ ) {
    corrida_charts_meteor_record *ch_met_rec_p;

    ch_met_rec_p = &(g_array_index (charts_meteors_list,
				    corrida_charts_meteor_record,
				    it));

    if ( met_num == ch_met_rec_p->met_no ) { /* meteor found */
      return ch_met_rec_p;
    }
  }

  return NULL; /* meteor not found */
}

gint get_velocity_estimation_type( GArray *meteors_list )
{
  gint it;
  corrida_meteor_record met_rec;
  gdouble vel;
  gdouble buf_1, buf_2;

  for ( it = 0; it < meteors_list->len; it++ ) {
    met_rec = g_array_index (meteors_list,
			     corrida_meteor_record,
			     it);
    vel = atof (met_rec.vel); 

    buf_1 = modf (vel, &buf_2);

    if ( fabs (buf_1) > 0.001 ) /* half type velocity found */
      return VELOCITY_ESTIMATION_TYPE_HALF;
      
  }

  /* half type velocity was not found  */
  return VELOCITY_ESTIMATION_TYPE_INT;
}

gint vel_rel2vel_deg( gdouble vel_rel, gint velocity_estimation_type)
{
  gint vel_deg;

  switch ( velocity_estimation_type ) {
  case VELOCITY_ESTIMATION_TYPE_HALF:
    if ( vel_rel < 0.1 )
      return 0;
    else if ( vel_rel < 1.1 )
      return 2;
    else if ( vel_rel < 1.6 )
      return 5;
    else if ( vel_rel < 2.1 )
      return 8;
    else if ( vel_rel < 2.6 )
      return 11;
    else if ( vel_rel < 3.1 )
      return 14;
    else if ( vel_rel < 3.6 )
      return 17;
    else if ( vel_rel < 4.1 )
      return 20;
    else if ( vel_rel < 4.6 )
      return 23;
    else if ( vel_rel < 5.1 )
      return 26;
    else if ( vel_rel < 6.1 )
      return 29;

    break;

  case VELOCITY_ESTIMATION_TYPE_INT:
    if ( vel_rel < 0.1 )
      return 0;
    else if ( vel_rel < 1.1 )
      return 3;
    else if ( vel_rel < 2.1 )
      return 8;
    else if ( vel_rel < 3.1 )
      return 13;
    else if ( vel_rel < 4.1 )
      return 18;
    else if ( vel_rel < 5.1 )
      return 23;
    else if ( vel_rel < 6.1 )
      return 26;

    break;
  }
  
  return 0;
}

void create_date_string (gchar *date_from_entry, gchar *date_to_meteors_files)
{
  /* input date format: 'dd/dd mm yy' 
   * output date format: 'yyyy mm dd/dd'
   */

  gint year;
  gint month;
  gint day_1, day_2;

  sscanf (date_from_entry, "%d/%d %d %d",
	  &day_1, &day_2, &month, &year);

  if ( year > 50 )
    year = year + 1900;
  else
    year = year + 2000;

  sprintf (date_to_meteors_files, "%04d %02d %02d/%02d",
	    year, month, day_1, day_2);
}

