#ifndef _FORMS_H
#define _FORMS_H

#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <math.h>

#include "console.h"
#include "misc.h"
#include "internal_data.h"

#define MAX_SITE_CODE_LEN 6
#define MAX_OBSERVER_CODE_LEN 6
#define MAX_VMDBSITE_STRLEN 300
#define PRINT_DATA_FROM_RUNS_LIST 1           /* if 0 then do not print */
#define PRINT_DATA_FROM_METEORS_LIST 1        /* if 0 then do not print */
#define PRINT_DATA_FROM_CHARTS_METEORS_LIST 1 /* if 0 then do not print */

/* columns for observations runs list */
enum {
  RUNS_LIST_COLUMN_BEGIN,
  RUNS_LIST_COLUMN_END,
  RUNS_LIST_COLUMN_TEFF,
  RUNS_LIST_COLUMN_LM,
  RUNS_LIST_COLUMN_F,
  RUNS_LIST_COLUMN_LAST_MET,
  RUNS_LIST_N_COLUMNS       
};

/* columns for meteors list */
enum {
  METEORS_LIST_COLUMN_NO,
  METEORS_LIST_COLUMN_TIME,
  METEORS_LIST_COLUMN_MAGNITUDE,
  METEORS_LIST_COLUMN_VELOCITY,
  METEORS_LIST_COLUMN_STREAM,
  METEORS_LIST_N_COLUMNS
};

/* columns for charts meteors list */
enum {
  CHARTS_METEORS_LIST_COLUMN_MET_NO,
  CHARTS_METEORS_LIST_COLUMN_MAP_NO,
  CHARTS_METEORS_LIST_COLUMN_CAR_BEG_X,
  CHARTS_METEORS_LIST_COLUMN_CAR_BEG_Y,
  CHARTS_METEORS_LIST_COLUMN_CAR_END_X,
  CHARTS_METEORS_LIST_COLUMN_CAR_END_Y,
  CHARTS_METEORS_LIST_COLUMN_EQU_BEG_ALPHA,
  CHARTS_METEORS_LIST_COLUMN_EQU_BEG_DELTA,
  CHARTS_METEORS_LIST_COLUMN_EQU_END_ALPHA,
  CHARTS_METEORS_LIST_COLUMN_EQU_END_DELTA,
  CHARTS_METEORS_LIST_N_COLUMNS
};

/* check types */
enum {
  CHECK_OBSERVER_CODE, /* imo observer code */
  CHECK_SITE_CODE,     /* imo site code */
  CHECK_DATE,          /* date */
  CHECK_FIELD_ALPHA,   /* alpha of the center of the observed fields */
  CHECK_FIELD_DELTA,   /* delta of the center of the observed fields */
  CHECK_TIME,          /* time begin, time end, meteor time */
  CHECK_TEFF,          /* teff */
  CHECK_MAG,           /* lm, meteors mag */
  CHECK_F,             /* f factor */
  CHECK_UINT,          /* last meteor, meteor number */
  CHECK_FLOAT,         /* coordinates */
  CHECK_VEL,           /* meteors velocities */
  CHECK_COLOR,         /* meteors colors */
  CHECK_STREAM          /* streams id */
};

/* velocity estimation type */
enum {
  VELOCITY_ESTIMATION_TYPE_HALF,
  VELOCITY_ESTIMATION_TYPE_INT
};

/* observations runs list */
typedef struct _corrida_observation_run {
  gchar *time_begin;  
  gchar *time_end;
  gchar *teff;
  gchar *lm;
  gchar *f;
  gchar *last_meteor;
} corrida_observation_run;

/* meteor record */
typedef struct _corrida_meteor_record {
  gchar *no;
  gchar *time;
  gchar *mag;
  gchar *vel;
  gchar *stream;
} corrida_meteor_record;

/* charts meteor record */
typedef struct _corrida_charts_meteor_record {
  gint met_no;
  gint map_no;
  cartesian_coords car_begin;
  cartesian_coords car_end;
  equatorial_coords_degs equ_begin;
  equatorial_coords_degs equ_end;
} corrida_charts_meteor_record;

/* export data */
typedef struct _corrida_export_data {
  GString *heads;
  GString *coords;
  GString *notpl;
  gboolean stat;
  
  /* additional data */
  gchar *observer_code;
  gchar *site_code;
  gint date_day;
  gint date_month;
  gint date_year;

  gchar *heads_filename;
  gchar *coords_filename;
  gchar *notpl_filename;
} corrida_export_data;

typedef struct _corrida_location_record {
  gchar *site_code;
  gdouble lmd;
  gchar lmd_dir;
  gdouble phi;
  gchar phi_dir;
} corrida_location_record;

void forms_init( int  *argc,
		 char ***argv );

void add_observation_run_record_to_garray( void );
void remove_observation_run_record_from_garray( gint index );
void clear_observations_runs_garray( void );

void add_meteor_record_to_garray( void );
void remove_meteor_record_from_garray( gint index );
void sort_corrida_meteors_in_garray( GArray *charts_meteors_list );
gint compare_corrida_meteor_record( gconstpointer a,
				    gconstpointer b );
void clear_meteors_garray( void );

void add_charts_meteor_record_to_garray( corrida_charts_meteor_record charts_met_rec);
void remove_charts_meteor_from_garray( gint index );
void sort_corrida_charts_meteors_in_garray( GArray *charts_meteors_list );
gint compare_corrida_charts_meteor_record( gconstpointer a,
					   gconstpointer b );
void clear_charts_meteors_garray( void );

int check_string (gchar *string, int check_type);

corrida_export_data create_export_data( void );

void print_data_from_runs_list( void );
void print_data_from_meteors_list( void );
void print_data_from_charts_meteors_list( void );

void get_location( gchar *site_code, gchar *location );

gdouble get_solar_longitude( gchar *input_date,
			     gint time_beg_h, gint time_beg_m,
			     gint time_end_h, gint time_end_m);
gchar *get_solar_long_record( gchar *date );
gchar *get_solar_long_record_from_internal_table( GArray *internal_solar_long_table,
						  gchar *date );
gchar *get_solar_long_record_from_file( gchar *date );
gdouble get_sol_from_record (gchar *s_rec, gint idx);
gchar *get_next_solar_long_record( gchar *date );
void get_next_date( gchar *previous_date, gchar *next_date );
void solar_longitude_not_found_message( gchar *date );


double get_center_alpha( gchar *f_alpha );
double get_center_delta( gchar *f_delta );

corrida_charts_meteor_record *get_meteor_from_charts_garray( GArray *charts_meteors_list,
							     gint met_num );
gint get_velocity_estimation_type( GArray *meteors_list );
gint vel_rel2vel_deg( gdouble vel_rel, gint velocity_estimation_type);

void create_date_string (gchar *date_from_entry, gchar *date_to_meteors_files);

#endif
