#include "gnomonic.h"

static gboolean gnomonic_init_flag = FALSE;
static charts_definitions charts[10];

/* gnomonic_init: initialize gnomonic module */
void gnomonic_init( void )
{
  if (gnomonic_init_flag != TRUE) {
    corrida_printf ("initializing gnomonic module...");

    charts[1].alpha_0 =  30.0;  charts[1].delta_0 = 55.68;
    charts[2].alpha_0 = 150.0;  charts[2].delta_0 = 55.68;
    charts[3].alpha_0 = 270.0;  charts[3].delta_0 = 55.68;
    charts[4].alpha_0 =  90.0;  charts[4].delta_0 =  4.89;
    charts[5].alpha_0 = 210.0;  charts[5].delta_0 =  4.89;
    charts[6].alpha_0 = 330.0;  charts[6].delta_0 =  4.89;
    charts[7].alpha_0 =  30.0;  charts[7].delta_0 = -4.89; 
    charts[8].alpha_0 = 150.0;  charts[8].delta_0 = -4.89;
    charts[9].alpha_0 = 270.0;  charts[9].delta_0 = -4.89;
    
    charts[1].X_0 = 175;        charts[1].Y_0 = 140;  
    charts[2].X_0 = 175;        charts[2].Y_0 = 140;  
    charts[3].X_0 = 175;        charts[3].Y_0 = 140;  
    charts[4].X_0 = 140;	charts[4].Y_0 = 175;  
    charts[5].X_0 = 140;	charts[5].Y_0 = 175;  
    charts[6].X_0 = 140;	charts[6].Y_0 = 175;  
    charts[7].X_0 = 175;	charts[7].Y_0 = 140;  
    charts[8].X_0 = 175;	charts[8].Y_0 = 140;  
    charts[9].X_0 = 175;	charts[9].Y_0 = 140;  

    charts[1].x_origin = 7;     charts[1].y_origin = 8;  
    charts[2].x_origin = 7;     charts[2].y_origin = 8;  
    charts[3].x_origin = 8;     charts[3].y_origin = 8;  
    charts[4].x_origin = 8;     charts[4].y_origin = 5;  
    charts[5].x_origin = 8;     charts[5].y_origin = 9;  
    charts[6].x_origin = 7;     charts[6].y_origin = 6;  
    charts[7].x_origin = 8;     charts[7].y_origin = 7;  
    charts[8].x_origin = 7;     charts[8].y_origin = 8;  
    charts[9].x_origin = 7;     charts[9].y_origin = 8;  

    charts[1].width = 974;      charts[1].height = 784;
    charts[2].width = 975;      charts[2].height = 784;
    charts[3].width = 968;      charts[3].height = 786;
    charts[4].width = 786;      charts[4].height = 970;
    charts[5].width = 786;      charts[5].height = 972;
    charts[6].width = 786;      charts[6].height = 972;
    charts[7].width = 970;      charts[7].height = 784;
    charts[8].width = 972;      charts[8].height = 784;
    charts[9].width = 974;      charts[9].height = 786;

    gnomonic_init_flag = TRUE;
  } else {
    corrida_printf ("gnomonic module already working");
  }
}

/* car2eq: convert pixel values from scanned Atlas Brno
 *         chart into equatorial coordinates
 */
int car2eq( char chartnum, 
	    cartesian_coords carcoords,
	    equatorial_coords_degs *eqcoords )
{
  float X_mm, Y_mm;
  float x, y;
  float R = 160.43;
  float t_radius;
  float vec_dir_p, vec_dir_q, vec_dir_r; /* vector direction (www.imo.net) */
    
  float alpha_0 = charts[chartnum].alpha_0;
  float delta_0 = charts[chartnum].delta_0;
  float X_0     = charts[chartnum].X_0;
  float Y_0     = charts[chartnum].Y_0;

  float X_width = 2.0 * X_0;
  float Y_width = 2.0 * Y_0;

  int x_origin  = charts[chartnum].x_origin;
  int y_origin  = charts[chartnum].y_origin;
  int width     = charts[chartnum].width;
  int height    = charts[chartnum].height;

  X_mm = ( X_width * (carcoords.x - x_origin) ) / width;
  Y_mm = ( Y_width * (height - (carcoords.y - y_origin)) ) / height;

  x = (X_mm - X_0) / R;
  y = (Y_mm - Y_0) / R;
  if (fabs (x) < 0.0005)
    x = 0.0005;
  if (fabs (y) < 0.0005)
    y = 0.0005;

  delta_0 = corrida_deg2rad (delta_0);
  alpha_0 = corrida_deg2rad (alpha_0);

  t_radius = sqrt (1 + x*x + y*y);
  vec_dir_p = (sin (delta_0) + y * cos (delta_0)) / t_radius;
  vec_dir_q = - x / t_radius;
  vec_dir_r = (cos (delta_0) - y * sin (delta_0)) / t_radius;

  eqcoords->delta = asin (vec_dir_p);
  if (vec_dir_q >= 0) {
    eqcoords->alpha = (acos (vec_dir_r / cos (eqcoords->delta)) + alpha_0);
  } else {
    eqcoords->alpha = 2.0*G_PI - acos (vec_dir_r / cos (eqcoords->delta)) + alpha_0;
  }

  if (eqcoords->alpha >= 2*G_PI)
    eqcoords->alpha = eqcoords->alpha - 2*G_PI;

  /* convert radians to degress */
  eqcoords->alpha = corrida_rad2deg (eqcoords->alpha);
  eqcoords->delta = corrida_rad2deg (eqcoords->delta);

  return 1;
}
