#ifndef _GUI_H_
#define _GUI_H_

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include "console.h"
#include "gnomonic.h"
#include "misc.h"
#include "forms.h"
#include "internal_data.h"

#define MAX_TAB_LABEL_LEN 50
#define MAX_CHART_NAME_LEN 50

#define GUI_BORDER_SIZE_SMALL 5
#define GUI_BORDER_SIZE_MEDIUM 10
#define GUI_BORDER_SIZE_LARGE 10

enum {
  ADD_RUN_BUTTON,
  REMOVE_RUN_BUTTON,
  ADD_METEOR_BUTTON,
  REMOVE_METEOR_BUTTON,
  EDIT_NEXT_RUN_CELL_BUTTON,
  EDIT_PREVIOUS_RUN_CELL_BUTTON,
  EDIT_NEXT_METEOR_CELL_BUTTON,
  EDIT_PREVIOUS_METEOR_CELL_BUTTON
};

enum {
  OBSERVATIONS_RUNS_TREEVIEW,
  METEORS_TREEVIEW
};

enum { /* this is widget num_pad, not keyborard entity */
  NUM_PAD_BUTTON_0,
  NUM_PAD_BUTTON_1,
  NUM_PAD_BUTTON_2,
  NUM_PAD_BUTTON_3,
  NUM_PAD_BUTTON_4,
  NUM_PAD_BUTTON_5,
  NUM_PAD_BUTTON_6,
  NUM_PAD_BUTTON_7,
  NUM_PAD_BUTTON_8,
  NUM_PAD_BUTTON_9,
  NUM_PAD_BUTTON_DEL,
  NUM_PAD_BUTTON_ENTER
};

enum {
  CHARTS_EDITION_STATUS_NULL,
  CHARTS_EDITION_STATUS_FIRST_POINT,
  CHARTS_EDITION_STATUS_SECOND_POINT
};

int corrida_gui_init( int  *argc,
		      char ***argv );

int corrida_gui_start( void );

void corrida_quit( GtkDialog *dialog,
		   gint arg1,
		   gpointer user_data );

void corrida_confirm_quit( GtkWidget *widget,
			   gpointer  callback_data);

GtkWidget *create_menu_bar( void );

GtkWidget *create_toolbar( void );

GtkWidget *create_observation( void );
void clear_observation( void );

GtkWidget *create_observations_runs_list( void );

GtkWidget *create_observation_runs_list_treeview( void );

GtkTreeModel *create_observations_runs_model( GArray *runs );

void add_columns_to_observations_runs( GtkTreeView *treeview );

GtkWidget *create_meteors_list( void );

GtkWidget *create_meteors_list_treeview( void );

GtkTreeModel *create_meteors_list_model (GArray *meteors);

void add_columns_to_meteors_list( GtkTreeView *treeview );

GtkWidget *create_statusbar( void );

GtkWidget *create_charts_notebook( void );

GtkWidget *create_charts_tools( void );

GtkWidget *create_charts_pointer_coords( void );

GtkWidget *create_charts_meteor_coords( void );

GtkWidget *create_charts_tools_num_pad( void );

GtkWidget *create_charts_meteors_list( void );

GtkWidget *create_charts_meteors_list_treeview( void );

GtkTreeModel *create_charts_meteors_list_model( GArray *meteors );

void add_columns_to_charts_meteors_list( GtkTreeView *treeview );

void corrida_error_box( GtkWindow *window,
			char *message );

static gint refresh_coords( GtkWidget *widget,
			    GdkEventMotion *event );

void charts_clear_entries( void );

void add_observation_run_record( void );
void add_observation_run_record_to_treeview( void );
void remove_observation_run_record( void );
void remove_observation_run_record_from_treeview( GtkListStore *list_store, 
						  GtkTreeIter  *tree_iter );
void clear_observations_runs_treeview( void );
void refresh_observations_runs_treeview( void );

void add_meteor_record( void );
void add_meteor_record_to_treeview( void );
void remove_meteor_record( void );
void remove_meteor_record_from_treeview( GtkListStore *list_store, 
					 GtkTreeIter  *tree_iter );
void clear_meteors_treeview( void );
void refresh_meteors_treeview( void );

void add_charts_meteor_record( corrida_charts_meteor_record charts_met_rec );
void add_charts_meteor_record_to_treeview( void );
void remove_charts_meteor_record( void );
void remove_charts_meteor_from_treeview( GtkListStore *list_store, 
					 GtkTreeIter  *tree_iter );
void clear_charts_meteors_treeview( void );
void refresh_charts_meteors_treeview( void );

void refresh_chart( void );
gboolean check_if_notebook_was_switched(gpointer data);

void edit_cell( gpointer data );

void statusbar_printf( char *message);

void erease_form( void );

void create_new_form_window( void );
void create_open_form_window( void );
void create_save_form_window( void );

void create_export_form_window( corrida_export_data export_data );

void create_export_preview_window( corrida_export_data export_data );

void destroy_file_selector( GtkButton *button,
			    gpointer data );

GdkPixbuf *get_corrida_logo( void );

/* callback functions: these functions should be defined
 *                     in gui_callbacks.c
 */

void new_form_cb( GtkWidget *widget,
		  gpointer data );

void open_form_cb( GtkWidget *widget,
		   gpointer data );

void save_form_cb( GtkWidget *widget,
		   gpointer data );

void export_preview_cb( GtkWidget *widget,
			gpointer data );

void export_form_cb( GtkWidget *widget,
		     gpointer data );

void export_form_cancel_pressed_cb( GtkButton *button,
				    gpointer data );

void export_form_export_pressed_cb( GtkButton *button,
				    gpointer data );

void menubar_help_clicked_cb( GtkMenuItem *menuitem,
			      gpointer user_data );

void entry_edited_cb( GtkEntry *entry,
		      gpointer user_data);

void observations_runs_cell_edit( GtkCellRendererText *cell,
				    const gchar       *path_string,
				    gchar             *new_text,
				    gpointer           data);

void meteors_list_cell_edit( GtkCellRenderer *cell,
			     const gchar     *path_string,
			     gchar           *new_text,
			     gpointer         data);

void menubar_about_clicked_cb( GtkMenuItem *menuitem,
			       gpointer user_data );

void button_clicked_cb (GtkButton *button,
			      gpointer   which_button);

gint treeview_key_press_cb( GtkWidget   *widget,
			    GdkEventKey *event,
			    gpointer     data);

void current_observations_runs_cell_set_cb( GtkCellRendererText *cell,
					    const gchar         *path_string,
					    gchar               *new_text,
					    gpointer             data);
GtkCellRendererText *current_observations_runs_cell_get_cb( void ); /* in fact it is not callback */

void current_meteors_cell_set_cb( GtkCellRendererText *cell,
				  const gchar         *path_string,
				  gchar               *new_text,
				  gpointer             data);
GtkCellRendererText *current_meteors_cell_get_cb( void ); /* in fact it is not callback */

void num_pad_button_clicked( GtkButton *button,
			     gpointer data );

gint chart_expose_event( GtkWidget      *widget,
			 GdkEventExpose *event );

gint chart_configure_event( GtkWidget         *widget,
			    GdkEventConfigure *event );

gint charts_style_check_button_pressed_cb( GtkWidget *widget,
					   gpointer   data );

gint chart_mouse_button_pressed_cb( GtkWidget      *widget,
				    GdkEventButton *event,
				    gpointer        data );

gint charts_accept_button_pressed_cb( GtkWidget *widget,
				      gpointer   data );

gint charts_clear_button_pressed_cb( GtkWidget *widget,
				     gpointer   data );

gint charts_remove_button_pressed_cb( GtkWidget *widget,
				      gpointer   data );

gint charts_sort_button_pressed_cb( GtkWidget *widget,
				    gpointer   data );

gint charts_notebook_switched_cb( GtkNotebook *notebook,
				  GtkNotebookPage *page,
				  guint page_num,
				  gpointer data);

void draw_meteor( GtkWidget *event_box,
		  corrida_charts_meteor_record meteor );

void draw_arrow( GdkPixmap *pixmap,
		 GdkGC *arrow_gc,
		 guint x1, guint y1, guint x2, guint y2,
		 char *string );

#endif
