#include "gui_charts.h"

GtkWidget *charts_notebook;
GtkWidget *charts_event_box[10];
GtkWidget *coords_alpha_label;
GtkWidget *coords_delta_label;
GtkWidget *charts_meteors_treeview;
GtkWidget *meteor_position_entry[4] = {NULL, NULL, NULL, NULL};
GtkWidget *num_pad_entry;

GtkWidget *create_charts_notebook( void )
{
  /* +------------------------+
   * |notebook...	      |
   * | +--------------------+ |
   * | |scrolled window	    | |
   * | | +----------------+ | |
   * | | |event box       | | |
   * | | | +------------+ | | |
   * | | | |drawing area| | | |
   * | | | +------------+ | | |
   * | | +----------------+ | |
   * | +--------------------+ |
   * +------------------------+
   */

  GtkWidget *charts_notebook;
  GtkWidget *chart_label;
  GtkWidget *scrolled_window;
  extern GtkWidget *charts_event_box[10];
  GtkWidget *drawing_area;
  GdkPixbuf *pixbuf;
  GdkPixmap *pixmap;

  gchar string_buf[MAX_CORRIDA_STRLEN];

  int it;
  char tab_label[MAX_TAB_LABEL_LEN];
  char chart_name[MAX_CHART_NAME_LEN];
  char chart_path[MAX_CHART_NAME_LEN];

  charts_notebook = gtk_notebook_new ();
  g_signal_connect (G_OBJECT (charts_notebook), "switch-page",
		    G_CALLBACK (charts_notebook_switched_cb), NULL);

  for (it = 1; it <= 9; it++) {
    scrolled_window = gtk_scrolled_window_new (NULL, NULL);
    gtk_widget_show (scrolled_window);

    charts_event_box[it] = gtk_event_box_new ();
    g_signal_connect (G_OBJECT (charts_event_box[it]), "motion_notify_event",
		      G_CALLBACK (refresh_coords), NULL);
    g_signal_connect (G_OBJECT (charts_event_box[it]), "button-press-event",
		      G_CALLBACK (chart_mouse_button_pressed_cb), 
		      (gpointer)it);
    gtk_widget_set_events (charts_event_box[it], 
			   GDK_POINTER_MOTION_MASK );
    gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled_window),
					   charts_event_box[it]);
    gtk_widget_show (charts_event_box[it]);

    /* creating chart to display */
    sprintf (chart_path, "%schart%d.jpg", internal_data_get_charts_path (), it);
    pixbuf = gdk_pixbuf_new_from_file (chart_path, NULL);

    if ( pixbuf == NULL) { /* error handling */
      sprintf (string_buf, "ERROR: unable to load image: %s", chart_path);
      corrida_printf (string_buf);
    } else { /* pixbuf succesfully loaded */
      drawing_area = gtk_drawing_area_new ();
      /* signals to handle backing pixmaps */
      g_signal_connect (G_OBJECT (drawing_area), "expose_event",
			G_CALLBACK (chart_expose_event), NULL);
      g_signal_connect (G_OBJECT (drawing_area), "configure_event",
			G_CALLBACK (chart_configure_event), NULL);
      gtk_widget_set_events (drawing_area, GDK_EXPOSURE_MASK);

      gtk_widget_set_size_request (drawing_area,
				   gdk_pixbuf_get_width (GDK_PIXBUF (pixbuf)),
				   gdk_pixbuf_get_height (GDK_PIXBUF (pixbuf))); 
      g_object_set_data (G_OBJECT (drawing_area), "which_chart", 
			 (gpointer)it);
      g_object_set_data (G_OBJECT (drawing_area), "which_pixbuf", 
			 (gpointer)pixbuf);
      g_object_set_data (G_OBJECT (drawing_area), "initialized", 
			 (gpointer)FALSE);
      g_object_set_data (G_OBJECT (charts_event_box[it]), "drawing_area",
			 (gpointer)drawing_area);
      gtk_container_add (GTK_CONTAINER (charts_event_box[it]),
			 drawing_area); 
      gtk_widget_show (drawing_area);
    }
    
    sprintf (tab_label, "Chart %d", it);
    chart_label = gtk_label_new (tab_label);
    gtk_widget_show (chart_label);
    
    gtk_notebook_append_page (GTK_NOTEBOOK (charts_notebook),
			      scrolled_window,
			      chart_label);
  }

  return charts_notebook;
}

GtkWidget *create_charts_tools( void )
{
  /* +-----------------------------------------+
   * |charts_tools (vbox)                      |
   * | +-------------------------------------+ |
   * | |pointer coords (frame)               | |
   * | +-------------------------------------+ |
   * | +-------------------------------------+ |
   * | |meteor coords			     | |
   * | +-------------------------------------+ |
   * | +-------------------------------------+ |
   * | |hseparator                           | |
   * | +-------------------------------------+ |
   * | +-------------------------------------+ |
   * | |num_pad				     | |
   * | +-------------------------------------+ |
   * | +-------------------------------------+ |
   * | |hseparator  			     | |
   * | +-------------------------------------+ |
   * | +-------------------------------------+ |
   * | |button				     | |
   * | +-------------------------------------+ |
   * | +-------------------------------------+ |
   * | |button				     | |
   * | +-------------------------------------+ |
   * | +-------------------------------------+ |
   * | |button (packed at end)		     | |
   * | +-------------------------------------+ |
   * | +-------------------------------------+ |
   * | |button (packed at end)		     | |
   * | +-------------------------------------+ |
   * +-----------------------------------------+
   */

  GtkWidget *charts_tools;
  GtkWidget *pointer_coords;
  GtkWidget *separator;
  GtkWidget *meteor_coords;
  GtkWidget *hseparator;
  GtkWidget *num_pad;
  GtkWidget *button;

  /* chart tools is actually vbox */
  charts_tools = gtk_vbox_new (FALSE, 0);

  pointer_coords = create_charts_pointer_coords ();
  gtk_box_pack_start (GTK_BOX (charts_tools), pointer_coords,
		      FALSE, FALSE, 0);
  gtk_widget_show (pointer_coords);

  meteor_coords = create_charts_meteor_coords ();
  gtk_box_pack_start (GTK_BOX (charts_tools), meteor_coords,
		      FALSE, FALSE, 0);
  gtk_widget_show (meteor_coords);

  hseparator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (charts_tools), hseparator,
		      FALSE, TRUE, 5);
  gtk_widget_show (hseparator);

  num_pad = create_charts_tools_num_pad ();
  gtk_box_pack_start (GTK_BOX (charts_tools), num_pad,
		      FALSE, FALSE, 0);
  gtk_widget_show (num_pad);

  hseparator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (charts_tools), hseparator,
		      FALSE, TRUE, 5);
  gtk_widget_show (hseparator);

  button = gtk_button_new_with_label ("Accept");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (charts_accept_button_pressed_cb), NULL);
  gtk_box_pack_start (GTK_BOX (charts_tools), button,
		      FALSE, FALSE, 0);
  gtk_widget_show (button);

  button = gtk_button_new_with_label ("Clear");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (charts_clear_button_pressed_cb), NULL);
  gtk_box_pack_start (GTK_BOX (charts_tools), button,
		      FALSE, FALSE, 0);
  gtk_widget_show (button);

  button = gtk_button_new_with_label ("Remove");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (charts_remove_button_pressed_cb), NULL);
  gtk_box_pack_end (GTK_BOX (charts_tools), button,
		      FALSE, FALSE, 0);
  gtk_widget_show (button);
  
  button = gtk_button_new_with_label ("Sort");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (charts_sort_button_pressed_cb), NULL);
  gtk_box_pack_end (GTK_BOX (charts_tools), button,
		      FALSE, FALSE, 0);
  gtk_widget_show (button);

  return charts_tools;
}

GtkWidget *create_charts_pointer_coords( void )
{
  /* +--------------------------
   * |pointer_coords (frame)
   * | +----------------
   * | |table (3X2)
   * | | +-----+ +-----+ 
   * | | |label| |label|
   * | | +-----+ +-----+
   * | | +-----+ +-----+
   * | | |label| |label|
   * | | +-----+ +-----+
   * | | +-------------+
   * | | |button       |
   * | | +-------------+
   * | +------------------
   * +---------------------
   */
  GtkWidget *pointer_coords;
  GtkWidget *table;
  GtkWidget *label;
  extern GtkWidget *coords_alpha_label;
  extern GtkWidget *coords_delta_label;
  GtkWidget *check_button;

  pointer_coords = gtk_frame_new ("Coords");

  table = gtk_table_new (3, 2, TRUE);
  gtk_container_add (GTK_CONTAINER (pointer_coords), table);
  gtk_widget_show (table);

  label = gtk_label_new ("Alpha:");
  gtk_table_attach_defaults (GTK_TABLE (table), label,
			     0, 1, 0, 1);
  gtk_widget_show (label);

  coords_alpha_label = gtk_label_new ("\0");
  gtk_table_attach_defaults (GTK_TABLE (table), coords_alpha_label,
			     1, 2, 0, 1);
  gtk_widget_show (coords_alpha_label);

  label = gtk_label_new ("Delta:");
  gtk_table_attach_defaults (GTK_TABLE (table), label,
			     0, 1, 1, 2);
  gtk_widget_show (label);

  coords_delta_label = gtk_label_new ("\0");
  gtk_table_attach_defaults (GTK_TABLE (table), coords_delta_label,
			     1, 2, 1, 2);
  gtk_widget_show (coords_delta_label);

  check_button = gtk_check_button_new_with_label ("Hours style");
  g_signal_connect (G_OBJECT (check_button), "clicked",
		    G_CALLBACK (charts_style_check_button_pressed_cb), NULL);
  gtk_table_attach_defaults (GTK_TABLE (table), check_button,
			     0, 2, 2, 3);
  gtk_widget_show (check_button);

  return pointer_coords;
}

GtkWidget *create_charts_meteor_coords( void )
{
  /* +---------------------------+
   * |meteor_coords              |
   * |(table 6x3)                |
   * | +-----------------------+ |
   * | |label                  | |
   * | +-----------------------+ |
   * | +-----+ +---------------+ |
   * | |label| |entry          | |
   * | +-----+ +---------------+ |
   * | +-----+ +---------------+ |
   * | |label| |entry          | |
   * | +-----+ +---------------+ |
   * | +-----------------------+ |
   * | |label                  | |
   * | +-----------------------+ |
   * | +-----+ +---------------+ |
   * | |label| |entry          | |
   * | +-----+ +---------------+ |
   * | +-----+ +---------------+ |
   * | |label| |entry          | |
   * | +-----+ +---------------+ |
   * +---------------------------+
   */

  GtkWidget *meteor_coords;
  GtkWidget *label;
  extern GtkWidget *meteor_position_entry[4];

  meteor_coords = gtk_table_new (6, 3, TRUE);
  
  label = gtk_label_new ("Begin");
  gtk_table_attach_defaults (GTK_TABLE (meteor_coords), label,
			     0, 3, 0, 1);
  gtk_widget_show (label);

  label = gtk_label_new ("alpha:");
  gtk_table_attach_defaults (GTK_TABLE (meteor_coords), label,
			     0, 1, 1, 2);
  gtk_widget_show (label);

  meteor_position_entry[0] = gtk_entry_new ();
  gtk_entry_set_max_length (GTK_ENTRY (meteor_position_entry[0]), 10);
  gtk_table_attach_defaults (GTK_TABLE (meteor_coords), meteor_position_entry[0],
			     1, 3, 1, 2);
  gtk_widget_show (meteor_position_entry[0]);
  
  label = gtk_label_new ("delta:");
  gtk_table_attach_defaults (GTK_TABLE (meteor_coords), label,
			     0, 1, 2, 3);
  gtk_widget_show (label);

  meteor_position_entry[1] = gtk_entry_new ();
  gtk_entry_set_max_length (GTK_ENTRY (meteor_position_entry[1]), 10);
  gtk_table_attach_defaults (GTK_TABLE (meteor_coords), meteor_position_entry[1],
			     1, 3, 2, 3);
  gtk_widget_show (meteor_position_entry[1]);

  label = gtk_label_new ("End");
  gtk_table_attach_defaults (GTK_TABLE (meteor_coords), label,
			     0, 3, 3, 4);
  gtk_widget_show (label);

  label = gtk_label_new ("alpha:");
  gtk_table_attach_defaults (GTK_TABLE (meteor_coords), label,
			     0, 1, 4, 5);
  gtk_widget_show (label);

  meteor_position_entry[2] = gtk_entry_new ();
  gtk_entry_set_max_length (GTK_ENTRY (meteor_position_entry[2]), 10);
  gtk_table_attach_defaults (GTK_TABLE (meteor_coords), meteor_position_entry[2],
			     1, 3, 4, 5);
  gtk_widget_show (meteor_position_entry[2]);
  
  label = gtk_label_new ("delta:");
  gtk_table_attach_defaults (GTK_TABLE (meteor_coords), label,
			     0, 1, 5, 6);
  gtk_widget_show (label);

  meteor_position_entry[3] = gtk_entry_new ();
  gtk_entry_set_max_length (GTK_ENTRY (meteor_position_entry[3]), 10);
  gtk_table_attach_defaults (GTK_TABLE (meteor_coords), meteor_position_entry[3],
			     1, 3, 5, 6);
  gtk_widget_show (meteor_position_entry[3]);


  return meteor_coords;
}

GtkWidget *create_charts_tools_num_pad( void )
{
  /* +-------------------------+
   * |num_pad (table)          |
   * | +-----+ +-------------+ |
   * | |label| |entry        | |
   * | +-----+ +-------------+ |
   * | +-----+ +-----+ +-----+ | 
   * | |  7  | |  8  | |  9  | |
   * | +-----+ +-----+ +-----+ |
   * | +-----+ +-----+ +-----+ |
   * | |  4  | |  5  | |  6  | |
   * | +-----+ +-----+ +-----+ |
   * | +-----+ +-----+ +-----+ |
   * | |  1  | |  2  | |  3  | |
   * | +-----+ +-----+ +-----+ |
   * | +-----+ +-------------+ |
   * | |  0  | |del          | |
   * | +-----+ +-------------+ |
   * +-------------------------+
   */		     

  GtkWidget *num_pad;
  GtkWidget *label;
  extern GtkWidget *num_pad_entry;
  GtkWidget *button;

  num_pad = gtk_table_new (5, 3, TRUE);

  /* label */
  label = gtk_label_new ("no.");
  gtk_table_attach_defaults (GTK_TABLE (num_pad), label,
			     0, 1, 0, 1);
  gtk_widget_show (label);

  /* num pad entry */
  num_pad_entry = gtk_entry_new ();
  gtk_entry_set_max_length (GTK_ENTRY (num_pad_entry), 10); /* you'd better don't
							     * change this value
							     */
  gtk_table_attach_defaults (GTK_TABLE (num_pad), num_pad_entry,
			     1, 3, 0, 1);
  gtk_widget_show (num_pad_entry); 
  
  /* '0' button */
  button = gtk_button_new_with_label ("0");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (num_pad_button_clicked), num_pad_entry);
  g_object_set_data (G_OBJECT (button),
		     "which_button",  (gint *)NUM_PAD_BUTTON_0);
  gtk_table_attach_defaults (GTK_TABLE (num_pad), button,
			     0, 1, 4, 5);
  gtk_widget_show (button);

  /* 'del' button */
  button = gtk_button_new_with_label ("Del");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (num_pad_button_clicked), num_pad_entry);
  g_object_set_data (G_OBJECT (button),
		     "which_button",  (gint *)NUM_PAD_BUTTON_DEL);
  gtk_table_attach_defaults (GTK_TABLE (num_pad), button,
			     1, 3, 4, 5);
  gtk_widget_show (button);

  /* '1' button */
  button = gtk_button_new_with_label ("1");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (num_pad_button_clicked), num_pad_entry);
  g_object_set_data (G_OBJECT (button),
		     "which_button",  (gint *)NUM_PAD_BUTTON_1);
  gtk_table_attach_defaults (GTK_TABLE (num_pad), button,
			     0, 1, 3, 4);
  gtk_widget_show (button);

  /* '2' button */
  button = gtk_button_new_with_label ("2");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (num_pad_button_clicked), num_pad_entry);
  g_object_set_data (G_OBJECT (button),
		     "which_button",  (gint *)NUM_PAD_BUTTON_2);
  gtk_table_attach_defaults (GTK_TABLE (num_pad), button,
			     1, 2, 3, 4);
  gtk_widget_show (button);
  
  /* '3' button */
  button = gtk_button_new_with_label ("3");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (num_pad_button_clicked), num_pad_entry);
  g_object_set_data (G_OBJECT (button),
		     "which_button",  (gint *)NUM_PAD_BUTTON_3);
  gtk_table_attach_defaults (GTK_TABLE (num_pad), button,
			     2, 3, 3, 4);
  gtk_widget_show (button); 

  /* '4' button */
  button = gtk_button_new_with_label ("4");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (num_pad_button_clicked), num_pad_entry);
  g_object_set_data (G_OBJECT (button),
		     "which_button",  (gint *)NUM_PAD_BUTTON_4);
  gtk_table_attach_defaults (GTK_TABLE (num_pad), button,
			     0, 1, 2, 3);
  gtk_widget_show (button);

  /* '5' button */
  button = gtk_button_new_with_label ("5");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (num_pad_button_clicked), num_pad_entry);
  g_object_set_data (G_OBJECT (button),
		     "which_button",  (gint *)NUM_PAD_BUTTON_5);
  gtk_table_attach_defaults (GTK_TABLE (num_pad), button,
			     1, 2, 2, 3);
  gtk_widget_show (button);
		    
  /* '6' button */
  button = gtk_button_new_with_label ("6");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (num_pad_button_clicked), num_pad_entry);
  g_object_set_data (G_OBJECT (button),
		     "which_button",  (gint *)NUM_PAD_BUTTON_6);
  gtk_table_attach_defaults (GTK_TABLE (num_pad), button,
			     2, 3, 2, 3);
  gtk_widget_show (button); 

  /* '7' button */
  button = gtk_button_new_with_label ("7");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (num_pad_button_clicked), num_pad_entry);
  g_object_set_data (G_OBJECT (button),
		     "which_button",  (gint *)NUM_PAD_BUTTON_7);
  gtk_table_attach_defaults (GTK_TABLE (num_pad), button,
			     0, 1, 1, 2);
  gtk_widget_show (button);

  /* '8' button */
  button = gtk_button_new_with_label ("8");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (num_pad_button_clicked), num_pad_entry);
  g_object_set_data (G_OBJECT (button),
		     "which_button",  (gint *)NUM_PAD_BUTTON_8);
  gtk_table_attach_defaults (GTK_TABLE (num_pad), button,
			     1, 2, 1, 2);
  gtk_widget_show (button);
  
  /* '9' button */
  button = gtk_button_new_with_label ("9");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (num_pad_button_clicked), num_pad_entry);
  g_object_set_data (G_OBJECT (button),
		     "which_button",  (gint *)NUM_PAD_BUTTON_9);
  gtk_table_attach_defaults (GTK_TABLE (num_pad), button,
			     2, 3, 1, 2);
  gtk_widget_show (button); 

  return num_pad;
}

GtkWidget *create_charts_meteors_list( void )
{
  /* +---------------------------+
   * |charts_meteors_list (frame)|
   * | +---------------+         |
   * | |scrolled_window|         |
   * | | +--------+    |         |
   * | | |treeview|    |         |
   * | | +--------+    |         |
   * | +---------------+         |
   * +---------------------------+
   */

  GtkWidget *charts_meteors_list;
  GtkWidget *scrolled_window;
  GtkWidget *treeview;

  charts_meteors_list = gtk_frame_new ("Charts meteors list");

  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_container_set_border_width (GTK_CONTAINER (scrolled_window),
				  GUI_BORDER_SIZE_SMALL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_AUTOMATIC,
				  GTK_POLICY_ALWAYS);
  gtk_container_add (GTK_CONTAINER (charts_meteors_list), scrolled_window);
  gtk_widget_show (scrolled_window);

  treeview = create_charts_meteors_list_treeview ();
  gtk_container_add (GTK_CONTAINER (scrolled_window),
		     treeview);
  gtk_widget_show (treeview);

  return charts_meteors_list;
}

GtkWidget *create_charts_meteors_list_treeview( void )
{
  extern GtkWidget *charts_meteors_treeview;
  extern GArray *charts_meteors_list;
  GtkTreeModel *model;

  model = create_charts_meteors_list_model (charts_meteors_list);
  charts_meteors_treeview = gtk_tree_view_new_with_model (model);
  g_object_unref (model);
  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (charts_meteors_treeview), TRUE);
  add_columns_to_charts_meteors_list (GTK_TREE_VIEW (charts_meteors_treeview));

  return charts_meteors_treeview;
}

GtkTreeModel *create_charts_meteors_list_model( GArray *charts_meteors )
{
  gint iter;
  GtkListStore *store;
  GtkTreeIter tree_iter;
  gchar string[100];

  /* create list store */
  store = gtk_list_store_new (CHARTS_METEORS_LIST_N_COLUMNS,
			      G_TYPE_STRING,
			      G_TYPE_STRING,
			      G_TYPE_STRING,
			      G_TYPE_STRING,
			      G_TYPE_STRING,
			      G_TYPE_STRING,
			      G_TYPE_STRING,
			      G_TYPE_STRING,
			      G_TYPE_STRING,
			      G_TYPE_STRING);

  /* add data to the list store */
  for ( iter = 0; iter < charts_meteors->len; iter++ ) {
    corrida_charts_meteor_record charts_met_record;

    charts_met_record = g_array_index (charts_meteors,
				       corrida_charts_meteor_record,
				       iter);
    gtk_list_store_append (store, &tree_iter);
    
    sprintf (string, "%d", charts_met_record.met_no);
    gtk_list_store_set (store, &tree_iter,
			CHARTS_METEORS_LIST_COLUMN_MET_NO, string, -1);

    sprintf (string, "%d", charts_met_record.map_no);
    gtk_list_store_set (store, &tree_iter,
			CHARTS_METEORS_LIST_COLUMN_MAP_NO, string, -1);

    sprintf (string, "%6.2f", charts_met_record.car_begin.x);
    gtk_list_store_set (store, &tree_iter,
			CHARTS_METEORS_LIST_COLUMN_CAR_BEG_X, string, -1);

    sprintf (string, "%6.2f", charts_met_record.car_begin.y);
    gtk_list_store_set (store, &tree_iter,
			CHARTS_METEORS_LIST_COLUMN_CAR_BEG_Y, string, -1);

    sprintf (string, "%6.2f", charts_met_record.car_end.x);
    gtk_list_store_set (store, &tree_iter,
			CHARTS_METEORS_LIST_COLUMN_CAR_END_X, string, -1);
    
    sprintf (string, "%6.2f", charts_met_record.car_end.y);
    gtk_list_store_set (store, &tree_iter,
			CHARTS_METEORS_LIST_COLUMN_CAR_END_Y, string, -1);
    
    sprintf (string, "%6.2f", charts_met_record.equ_begin.alpha);
    gtk_list_store_set (store, &tree_iter,
			CHARTS_METEORS_LIST_COLUMN_EQU_BEG_ALPHA, string, -1);

    sprintf (string, "%6.2f", charts_met_record.equ_begin.delta);
    gtk_list_store_set (store, &tree_iter,
			CHARTS_METEORS_LIST_COLUMN_EQU_BEG_DELTA, string, -1);

    sprintf (string, "%6.2f", charts_met_record.equ_end.alpha);
    gtk_list_store_set (store, &tree_iter,
			CHARTS_METEORS_LIST_COLUMN_EQU_END_ALPHA, string, -1);

    sprintf (string, "%6.2f", charts_met_record.equ_end.delta);
    gtk_list_store_set (store, &tree_iter,
			CHARTS_METEORS_LIST_COLUMN_EQU_END_DELTA, string, -1);

  }

  return GTK_TREE_MODEL (store);
}

void add_columns_to_charts_meteors_list( GtkTreeView *treeview )
{
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;
  GtkTreeModel *model = gtk_tree_view_get_model (treeview);

  /* column for meteor number */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *)CHARTS_METEORS_LIST_COLUMN_MET_NO); 
  column = gtk_tree_view_column_new_with_attributes ("No.",
						     renderer,
						     "text",
						     CHARTS_METEORS_LIST_COLUMN_MET_NO,
						     NULL);
  g_object_set (G_OBJECT (column),
		"resizable", TRUE,
		"min-width", 35,
		NULL);
  gtk_tree_view_append_column (treeview, column);
		
		
  /* column for racbeg */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *)CHARTS_METEORS_LIST_COLUMN_EQU_BEG_ALPHA);
  column = gtk_tree_view_column_new_with_attributes ("Rac. beg.",
						     renderer,
						     "text",
						     CHARTS_METEORS_LIST_COLUMN_EQU_BEG_ALPHA,
						     NULL);
  g_object_set (G_OBJECT (column),
		"resizable", TRUE,
		"min-width", 50,
		NULL);
  gtk_tree_view_append_column (treeview, column);

  /* column for decbeg */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *)CHARTS_METEORS_LIST_COLUMN_EQU_BEG_DELTA);
  column = gtk_tree_view_column_new_with_attributes ("Dec. beg.",
						     renderer,
						     "text",
						     CHARTS_METEORS_LIST_COLUMN_EQU_BEG_DELTA ,
						     NULL);
  g_object_set (G_OBJECT (column),
		"resizable", TRUE,
		"min-width", 50,
		NULL);
  gtk_tree_view_append_column (treeview, column); 

  /* column for recend */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *)CHARTS_METEORS_LIST_COLUMN_EQU_END_ALPHA);
  column = gtk_tree_view_column_new_with_attributes ("Rac. end",
						     renderer,
						     "text",
						     CHARTS_METEORS_LIST_COLUMN_EQU_END_ALPHA,
						     NULL);
  g_object_set (G_OBJECT (column),
		"resizable", TRUE,
		"min-width", 50,
		NULL);
  gtk_tree_view_append_column (treeview, column);

  /* column for decend */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *)CHARTS_METEORS_LIST_COLUMN_EQU_END_DELTA);
  column = gtk_tree_view_column_new_with_attributes ("Dec. end",
						     renderer,
						     "text",
						     CHARTS_METEORS_LIST_COLUMN_EQU_END_DELTA,
						     NULL);
  g_object_set (G_OBJECT (column),
		"resizable", TRUE,
		"min-width", 50,
		NULL);
  gtk_tree_view_append_column (treeview, column);

  /* column for map num */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *)CHARTS_METEORS_LIST_COLUMN_MAP_NO);
  column = gtk_tree_view_column_new_with_attributes ("Map",
						     renderer,
						     "text",
						     CHARTS_METEORS_LIST_COLUMN_MAP_NO,
						     NULL);
  g_object_set (G_OBJECT (column),
		"resizable", TRUE,
		"min-width", 50,
		NULL);
  gtk_tree_view_append_column (treeview, column);

}

void add_charts_meteor_record ( corrida_charts_meteor_record charts_met_rec )
{
  extern GtkWidget *charts_scrolled_window;
  extern GtkWidget *charts_meteors_treeview;

  /* add new record to GArray */
  add_charts_meteor_record_to_garray (charts_met_rec);
  /* add new record to treeview */
  add_charts_meteor_record_to_treeview ();

  print_data_from_charts_meteors_list ();
  corrida_printf ("charts meteor record added");
}

void add_charts_meteor_record_to_treeview( void )
{
  /* this function should be invoked only by 'add_charts_meteor_record ()'
   * after call of 'add_charts_meteor_record_to_garray ()'
   */

  extern GArray *charts_meteors_list;
  extern GtkWidget *charts_meteors_treeview;
  corrida_charts_meteor_record charts_met_rec;
  gint iter;
  GtkTreeIter tree_iter;
  GtkTreeModel *model;
  char string[100];

  iter = charts_meteors_list->len - 1; /* iter points on last element
					* of this array
					*/

  model = gtk_tree_view_get_model (GTK_TREE_VIEW (charts_meteors_treeview));

  charts_met_rec = g_array_index (charts_meteors_list,
				  corrida_charts_meteor_record,
				  iter);
  
  gtk_list_store_append (GTK_LIST_STORE (model), &tree_iter);

  /* add record to the list store */
  sprintf (string, "%d", charts_met_rec.met_no);
  gtk_list_store_set (GTK_LIST_STORE (model), &tree_iter,
		      CHARTS_METEORS_LIST_COLUMN_MET_NO, string, -1);

  sprintf (string, "%d", charts_met_rec.map_no);
  gtk_list_store_set (GTK_LIST_STORE (model), &tree_iter,
		      CHARTS_METEORS_LIST_COLUMN_MAP_NO, string, -1);

  sprintf (string, "%6.2f", charts_met_rec.car_begin.x);
  gtk_list_store_set (GTK_LIST_STORE (model), &tree_iter,
		      CHARTS_METEORS_LIST_COLUMN_CAR_BEG_X, string, -1);

  sprintf (string, "%6.2f", charts_met_rec.car_begin.y);
  gtk_list_store_set (GTK_LIST_STORE (model), &tree_iter,
		      CHARTS_METEORS_LIST_COLUMN_CAR_BEG_Y, string, -1);

  sprintf (string, "%6.2f", charts_met_rec.car_end.x);
  gtk_list_store_set (GTK_LIST_STORE (model), &tree_iter,
		      CHARTS_METEORS_LIST_COLUMN_CAR_END_X, string, -1);

  sprintf (string, "%6.2f", charts_met_rec.car_end.y);
  gtk_list_store_set (GTK_LIST_STORE (model), &tree_iter,
		      CHARTS_METEORS_LIST_COLUMN_CAR_END_Y, string, -1);
  
  sprintf (string, "%6.2f", charts_met_rec.equ_begin.alpha);
  gtk_list_store_set (GTK_LIST_STORE (model), &tree_iter,
		      CHARTS_METEORS_LIST_COLUMN_EQU_BEG_ALPHA, string, -1);
  
  sprintf (string, "%6.2f", charts_met_rec.equ_begin.delta);
  gtk_list_store_set (GTK_LIST_STORE (model), &tree_iter,
		      CHARTS_METEORS_LIST_COLUMN_EQU_BEG_DELTA, string, -1);

  sprintf (string, "%6.2f", charts_met_rec.equ_end.alpha);
  gtk_list_store_set (GTK_LIST_STORE (model), &tree_iter,
		      CHARTS_METEORS_LIST_COLUMN_EQU_END_ALPHA, string, -1);

  sprintf (string, "%6.2f", charts_met_rec.equ_end.delta);
  gtk_list_store_set (GTK_LIST_STORE (model), &tree_iter,
		      CHARTS_METEORS_LIST_COLUMN_EQU_END_DELTA, string, -1);

}

void remove_charts_meteor_record( void )
{
  extern GArray *charts_meteors_list;
  extern GtkWidget *charts_meteors_treeview;
  GtkTreeModel *model;
  GtkTreeSelection *selection;
  GtkTreeIter tree_iter;

  model = gtk_tree_view_get_model (GTK_TREE_VIEW (charts_meteors_treeview));
  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (charts_meteors_treeview));

  if (gtk_tree_selection_get_selected (selection, NULL, &tree_iter)) {
    gint iter;
    GtkTreePath *path;

    path = gtk_tree_model_get_path (model, &tree_iter);
    iter = gtk_tree_path_get_indices (path)[0];

    /* remove selected record from GArray */
    remove_charts_meteor_from_garray (iter);

    /* remove selected record from treeview */
    remove_charts_meteor_from_treeview (GTK_LIST_STORE (model), 
					&tree_iter);
    gtk_tree_path_free (path);

    corrida_printf ("charts meteor record removed");
  }
}

void remove_charts_meteor_from_treeview( GtkListStore *list_store, 
					 GtkTreeIter  *tree_iter )
{
  gtk_list_store_remove (list_store, tree_iter);
}

/* refresh_chart (): refresch active chart */
void refresh_chart( void )
{
  extern GtkWidget *charts_notebook;

  if ( charts_notebook != NULL ) { /* proces only if notebook exists */ 

    extern GtkWidget *charts_event_box[10];
    extern GArray *charts_meteors_list;
    GtkWidget *event_box;
    GtkWidget *drawing_area;
    GdkPixbuf *pixbuf;
    GdkPixmap *pixmap;
    gint page;
    gint it;
    
    page = gtk_notebook_get_current_page (GTK_NOTEBOOK (charts_notebook)) + 1;
    event_box = charts_event_box[page];
    drawing_area = g_object_get_data (G_OBJECT (event_box), "drawing_area");
    pixbuf = g_object_get_data (G_OBJECT (drawing_area), "which_pixbuf");
    pixmap = g_object_get_data (G_OBJECT (drawing_area), "which_pixmap");
    
    /* draw pixbuf on pixmap */
    gdk_draw_pixbuf (pixmap, NULL, pixbuf,
		     0, 0, 0, 0, -1, -1, GDK_RGB_DITHER_NORMAL, 2, 2);
    
    /* draw meteors on map */
    for ( it = 0; it < charts_meteors_list->len; it++ ) {
      corrida_charts_meteor_record charts_met_rec;
      gint map_no;
      
      charts_met_rec = g_array_index (charts_meteors_list, 
				      corrida_charts_meteor_record,
				      it);
      map_no = charts_met_rec.map_no;
      if ( map_no == page) { /* meteor from active map */
	draw_meteor (event_box, charts_met_rec);
      }
    }
    /* draw NULL meteor to refresh map if GArray is empty */
    if ( charts_meteors_list->len == 0 ) {
      corrida_charts_meteor_record charts_met_rec;
      gint map_no;
      
      charts_met_rec.met_no          = 0;
      charts_met_rec.map_no          = page;
      charts_met_rec.car_begin.x     = 0;
      charts_met_rec.car_begin.y     = 0;
      charts_met_rec.car_end.x       = 0;
      charts_met_rec.car_end.y       = 0;
      charts_met_rec.equ_begin.alpha = 0;
      charts_met_rec.equ_begin.delta = 0;
      charts_met_rec.equ_end.alpha   = 0;
      charts_met_rec.equ_end.delta   = 0;
      
      draw_meteor (event_box, charts_met_rec);
    }
  }
}

gboolean check_if_notebook_was_switched(gpointer data)
{
  extern GtkWidget *charts_notebook;
  static gint last_chart_num = 0;
  gint current_chart_num;

  current_chart_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (charts_notebook));

  if ( last_chart_num != current_chart_num ) {
    refresh_chart ();
    last_chart_num = current_chart_num;
  }

  return TRUE;
}

void clear_charts_meteors_treeview( void )
{
  extern GArray *charts_meteors_list;
  extern GtkWidget *charts_meteors_treeview; 
  GtkTreeModel *model;
  GtkTreeIter tree_iter;
  GtkTreePath *path;
  gint it;

  model = gtk_tree_view_get_model (GTK_TREE_VIEW (charts_meteors_treeview));
  path = gtk_tree_path_new_from_string ("0");

  while ( gtk_tree_model_get_iter (model, &tree_iter, path) ) {
    remove_charts_meteor_from_treeview (GTK_LIST_STORE (model),
					&tree_iter);
  }
  gtk_tree_path_free (path);
}

void refresh_charts_meteors_treeview( void )
{
  extern GArray *charts_meteors_list;
  extern GtkWidget *charts_meteors_treeview;
  corrida_charts_meteor_record charts_met_rec;
  gint iter;
  GtkTreeIter tree_iter;
  GtkTreeModel *model;
  gchar string[MAX_CORRIDA_STRLEN];

  model = gtk_tree_view_get_model (GTK_TREE_VIEW (charts_meteors_treeview));

  for ( iter = 0; iter < charts_meteors_list->len; iter++ ) {
    charts_met_rec = g_array_index (charts_meteors_list,
				    corrida_charts_meteor_record,
				    iter);

  gtk_list_store_append (GTK_LIST_STORE (model), &tree_iter);

  /* add record to the list store */
  sprintf (string, "%d", charts_met_rec.met_no);
  gtk_list_store_set (GTK_LIST_STORE (model), &tree_iter,
		      CHARTS_METEORS_LIST_COLUMN_MET_NO, string, -1);
  
  sprintf (string, "%d", charts_met_rec.map_no);
  gtk_list_store_set (GTK_LIST_STORE (model), &tree_iter,
		      CHARTS_METEORS_LIST_COLUMN_MAP_NO, string, -1);
  
  sprintf (string, "%6.2f", charts_met_rec.car_begin.x);
  gtk_list_store_set (GTK_LIST_STORE (model), &tree_iter,
		      CHARTS_METEORS_LIST_COLUMN_CAR_BEG_X, string, -1);
  
  sprintf (string, "%6.2f", charts_met_rec.car_begin.y);
  gtk_list_store_set (GTK_LIST_STORE (model), &tree_iter,
		      CHARTS_METEORS_LIST_COLUMN_CAR_BEG_Y, string, -1);
  
  sprintf (string, "%6.2f", charts_met_rec.car_end.x);
  gtk_list_store_set (GTK_LIST_STORE (model), &tree_iter,
		      CHARTS_METEORS_LIST_COLUMN_CAR_END_X, string, -1);
  
  sprintf (string, "%6.2f", charts_met_rec.car_end.y);
  gtk_list_store_set (GTK_LIST_STORE (model), &tree_iter,
		      CHARTS_METEORS_LIST_COLUMN_CAR_END_Y, string, -1);
  
  sprintf (string, "%6.2f", charts_met_rec.equ_begin.alpha);
  gtk_list_store_set (GTK_LIST_STORE (model), &tree_iter,
		      CHARTS_METEORS_LIST_COLUMN_EQU_BEG_ALPHA, string, -1);
  
  sprintf (string, "%6.2f", charts_met_rec.equ_begin.delta);
  gtk_list_store_set (GTK_LIST_STORE (model), &tree_iter,
		      CHARTS_METEORS_LIST_COLUMN_EQU_BEG_DELTA, string, -1);
  
  sprintf (string, "%6.2f", charts_met_rec.equ_end.alpha);
  gtk_list_store_set (GTK_LIST_STORE (model), &tree_iter,
		      CHARTS_METEORS_LIST_COLUMN_EQU_END_ALPHA, string, -1);
  
  sprintf (string, "%6.2f", charts_met_rec.equ_end.delta);
  gtk_list_store_set (GTK_LIST_STORE (model), &tree_iter,
		      CHARTS_METEORS_LIST_COLUMN_EQU_END_DELTA, string, -1);
  }
}

static gint refresh_coords( GtkWidget *widget,
			    GdkEventMotion *event )
{
  extern GtkWidget *charts_notebook;
  extern GtkWidget *coords_alpha_label;
  extern GtkWidget *coords_delta_label;
  gint chartnum;
  cartesian_coords carcoords;
  gint style;
  equatorial_coords_degs eqcoords_d;
  equatorial_coords_hours eqcoords_h;
  char coords_label_text[MAX_EQU_HOURS_STRING];
  
  /* get a chartnum to procces coords */
  chartnum =  1 + gtk_notebook_get_current_page (GTK_NOTEBOOK (charts_notebook));

  /* get pixels values of event */
  carcoords.x = (float) event->x;
  carcoords.y = (float) event->y;

  style = internal_data_get_charts_pointer_coords_style ();

  car2eq (chartnum, carcoords, &eqcoords_d);
  equ_degs2equ_hours (eqcoords_d, &eqcoords_h);

  switch ( style ) {
  case SETTINGS_CHARTS_POINTER_COORDS_STYLE_DEGS:
    {
      /* print alpha */
      sprintf (coords_label_text, "%5.2f deg", eqcoords_d.alpha);
      gtk_label_set_text (GTK_LABEL (coords_alpha_label),
			  coords_label_text);

      /* print delta */
      sprintf (coords_label_text, "%5.2f deg", eqcoords_d.delta);
      gtk_label_set_text (GTK_LABEL (coords_delta_label),
			  coords_label_text);

      break;
    }

  case SETTINGS_CHARTS_POINTER_COORDS_STYLE_HOURS:
    {
      /* print alpha */
      sprintf (coords_label_text, "%2dh %2dm", 
	       eqcoords_h.alpha_hour, eqcoords_h.alpha_min);
      gtk_label_set_text (GTK_LABEL (coords_alpha_label),
			  coords_label_text); 

      /* print delta */
      sprintf (coords_label_text, "%2dd %2.0fm",
	       eqcoords_h.delta_deg, fabs (eqcoords_h.delta_min));
      gtk_label_set_text (GTK_LABEL (coords_delta_label),
			  coords_label_text);

      break;
    }
  }

  return FALSE;
}

void charts_clear_entries( void )
{
  extern GtkWidget *meteor_position_entry[4];
  extern corrida_charts_meteor_record current_meteor;
  extern GtkWidget *num_pad_entry;
  corrida_charts_meteor_record meteor = current_meteor;
  gint it;

  for ( it = 0; it < 4; it++ ) { /* clear and activate meteor position entries */
    if (meteor_position_entry[it] != NULL) {
      gtk_editable_set_editable (GTK_EDITABLE (meteor_position_entry[it]),
				 TRUE);
      gtk_entry_set_text (GTK_ENTRY (meteor_position_entry[it]), "\0");
    }
  }

  /* clear meteor number entry */
  if (num_pad_entry != NULL)
    gtk_entry_set_text (GTK_ENTRY (num_pad_entry), "\0");

  /* clear current meteor data */
  current_meteor.met_no = -1;
  current_meteor.map_no = -1;
  current_meteor.car_begin.x = 0;
  current_meteor.car_begin.y = 0;
  current_meteor.car_end.x = 0;
  current_meteor.car_end.y = 0;
  current_meteor.equ_begin.alpha = 0;
  current_meteor.equ_begin.delta = 0;
  current_meteor.equ_end.alpha = 0;
  current_meteor.equ_end.delta = 0;

  corrida_printf ("entries cleared");
}

void draw_meteor( GtkWidget *event_box,
		  corrida_charts_meteor_record meteor )
{
  GtkWidget *drawing_area;
  GdkPixmap *pixmap;
  static GdkGC *gc = NULL;
  gchar met_no[11];
  gint size_x, size_y;

  drawing_area = g_object_get_data (G_OBJECT (event_box), "drawing_area");
  pixmap = g_object_get_data (G_OBJECT (drawing_area), "which_pixmap");

  if ( gc == NULL ) { /* initialize graphic context for plotted meteors */
    GdkColor meteor_color;
    GdkColormap *colormap;

    colormap = gdk_colormap_get_system ();
    
    meteor_color.red = 65535;
    meteor_color.green = 0;
    meteor_color.blue = 0;

    gdk_colormap_alloc_color (colormap,
			      &meteor_color,
			      TRUE,
			      TRUE);    

    gc = gdk_gc_new (pixmap);
    gdk_gc_set_foreground (gc, &meteor_color);
  }

  sprintf (met_no, "%d", meteor.met_no);

  draw_arrow (pixmap, gc,
	      meteor.car_begin.x, meteor.car_begin.y,
	      meteor.car_end.x, meteor.car_end.y,
	      met_no);

  gdk_draw_drawable (drawing_area->window,
		     drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)],
		     pixmap,
		     1, 1, 1, 1,
		     drawing_area->allocation.width,
		     drawing_area->allocation.height);
}

void draw_arrow( GdkPixmap *pixmap,
		 GdkGC *arrow_gc,
		 guint x1, guint y1, guint x2, guint y2,
		 char *string )
{
  float spike_length = 15.0; /* FIXME: set it in internal_data module */
  float spike_angle = 15.0 * G_PI / 180; /* FIXME: set it in internal_data module */
  float marker_length = 10.0; /* FIXME: set it in internal_data module */
  float string_length = 10.0; /* FIXME: set it in internal_data module */
  gint x = x2 - x1;
  gint y = y2 - y1;
  gint m1_x, m1_y, m2_x, m2_y; /* markers point */
  float spike_x, spike_y;
  float string_x, string_y;
  float alpha;
  GdkFont static *font = NULL ;

  if (font == NULL)
    font = gdk_font_load ("-adobe-helvetica-bold-r-normal--12-120-75-75-p-70-iso8859-1");
  
  alpha = atan ((float)y / (float)x);
  if ( x < 0 ) 
    alpha = alpha + G_PI;

  gdk_draw_line (pixmap,
		 arrow_gc,
		 x1, y1,
		 x2, y2);

  /* draw spike */
  spike_x = x2 - spike_length * cos (alpha - spike_angle);
  spike_y = y2 - spike_length * sin (alpha - spike_angle);
  gdk_draw_line (pixmap, arrow_gc, 
		 x2, y2, (gint)spike_x, (gint)spike_y);
  spike_x = x2 - spike_length * cos (alpha + spike_angle);
  spike_y = y2 - spike_length * sin (alpha + spike_angle);
  gdk_draw_line (pixmap, arrow_gc, 
		 x2, y2, (gint)spike_x, (gint)spike_y);
  
  /* draw marker at start point of arrow */
  m1_x = x1 + (marker_length / 2.0) * cos (alpha + G_PI / 2.0);
  m1_y = y1 + (marker_length / 2.0) * sin (alpha + G_PI / 2.0);
  m2_x = x1 + (marker_length / 2.0) * cos (alpha - G_PI / 2.0);
  m2_y = y1 + (marker_length / 2.0) * sin (alpha - G_PI / 2.0);
  gdk_draw_line (pixmap, arrow_gc,
		 m1_x, m1_y, m2_x, m2_y);

  /* draw the string if exist */
  if ( string != NULL && strlen (string) > 0 ) {
    gdouble fx1, fx2, fy1, fy2;
    gdouble vec1_x, vec1_y; /* wektor jednostkowy */
    gdouble vec_length;
    gdouble shiftx_factor; /* shift x of arrow label due to spike */
    gdouble shifty_factor; /* shift y of arrow label due to spike */
    gdouble angle;

    fx1 = (gdouble) x1;
    fx2 = (gdouble) x2;
    fy1 = (gdouble) y1;
    fy2 = (gdouble) y2;

    vec_length = sqrt (pow (fx2-fx1, 2) + pow (fy2-fy1, 2));

    vec1_x = (fx2-fx1) / (vec_length);
    vec1_y = (fy2-fy1) / (vec_length);

    angle = compute_angle (vec1_x, vec1_y);

    /* compute shif x */
    if ( (vec1_x > 0 && vec1_y > 0) || /* first of fourth quarter */
	 (vec1_x > 0 && vec1_y < 0) ) {
      shiftx_factor = 9 * sin (angle / 2.0);
    } else /* second or third quarter */ {
      shiftx_factor = 9 * strlen(string) * sin (angle / 2.0);
    }

    /* compute shift y */
    if ( angle >= 0 && angle <= G_PI ) {
      shifty_factor = 15 * sin (angle);
    } else {
      shifty_factor = - (4 * sin (angle));
    }
    
    string_x = fx2 + 2 + shiftx_factor * vec1_x;
    string_y = fy2 + shifty_factor * vec1_y;
    gdk_draw_string (pixmap,
		     font,
		     arrow_gc,
		     string_x, string_y,
		     string);

    
  }
}
