#include "gui.h"

void create_export_form_window( corrida_export_data export_data )
{
  /* +----------------------------+
   * |export_form_window          |
   * | +------------------------+ |
   * | |table (6x2)		| |
   * | | +--------------------+	| |
   * | | |label               |	| |
   * | | +--------------------+	| |
   * | | +-----+ +------------+	| |
   * | | |label| |main_entry  |	| |
   * | | +-----+ +------------+	| |
   * | | +-----+ +------------+	| |
   * | | |label| |heads_entry |	| |
   * | | +-----+ +------------+	| |
   * | | +-----+ +------------+	| |
   * | | |label| |coords_entry|	| |
   * | | +-----+ +------------+	| |
   * | | +-----+ +------------+	| |
   * | | |label| |notpl_entry |	| |
   * | | +-----+ +------------+ | |
   * | | +--------------------+ | |
   * | | |hbox                | | |
   * | | |  +------+ +------+ | | |
   * | | |  |button| |button| |	| |
   * | | |  +------+ +------+ |	| |
   * | | +--------------------+	| |
   * | +------------------------+ |
   * +----------------------------+
   */

  GtkWidget *export_form_window;
  GtkWidget *table;
  GtkWidget *label;
  GtkWidget *general_entry;
  GtkWidget *heads_entry;
  GtkWidget *coords_entry;
  GtkWidget *notpl_entry;
  GtkWidget *hbox;
  GtkWidget *button;

  gchar general_string[MAX_CORRIDA_STRLEN];
  gchar heads_string[MAX_CORRIDA_STRLEN];
  gchar coords_string[MAX_CORRIDA_STRLEN];
  gchar notpl_string[MAX_CORRIDA_STRLEN];

  static gchar *heads_filename = NULL;
  static gchar *coords_filename = NULL;
  static gchar *notpl_filename = NULL;
  static corrida_export_data private_export_data; /* its adress we
						   * we pass to export
						   * button callback
						   */

  /* initialize filenames */
  if ( heads_filename != NULL )
    g_free (heads_filename);
  if ( coords_filename != NULL )
    g_free (coords_filename);
  if ( notpl_filename != NULL )
    g_free (notpl_filename);      

  sprintf (general_string, "%02d%02d%02d-%s",
	   export_data.date_year, 
	   export_data.date_month,
	   export_data.date_day,
	   export_data.observer_code);
  sprintf (heads_string, "%s-heads.txt", general_string);
  sprintf (coords_string, "%s-coords.txt", general_string);
  sprintf (notpl_string, "%s-notpl.txt", general_string);

  export_form_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  g_signal_connect (G_OBJECT (export_form_window), "delete-event",
		    G_CALLBACK (gtk_widget_destroy), NULL);
  g_signal_connect (G_OBJECT (export_form_window), "destroy-event",
		    G_CALLBACK (gtk_widget_destroy), NULL);
  gtk_window_set_title (GTK_WINDOW (export_form_window),
			"corrida export form");

  table = gtk_table_new (5, 2, FALSE);
  gtk_container_set_border_width (GTK_CONTAINER (table),
				  GUI_BORDER_SIZE_SMALL);
  gtk_container_add (GTK_CONTAINER (export_form_window), table);
  gtk_widget_show (table);

  label = gtk_label_new ("Enter export filename(s):");
  gtk_table_attach_defaults (GTK_TABLE (table), label,
			     0, 2, 0, 1);
  gtk_widget_show (label);

  /* general field */
  label = gtk_label_new ("General:");
  gtk_table_attach_defaults (GTK_TABLE (table), label,
			     0, 1, 1, 2);
  gtk_widget_show (label);

  general_entry = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (general_entry),
		      general_string);
  gtk_table_attach_defaults (GTK_TABLE (table), general_entry,
			     1, 2, 1, 2);
  gtk_widget_show (general_entry);
  
  /* heads fields */
  label = gtk_label_new ("Headers filename:");
  gtk_table_attach_defaults (GTK_TABLE (table), label,
			     0, 1, 2, 3);
  gtk_widget_show (label);

  heads_entry = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (heads_entry),
		      heads_string);
  gtk_table_attach_defaults (GTK_TABLE (table), heads_entry,
			     1, 2, 2, 3);
  gtk_widget_show (heads_entry);

  /* coords fields */
  label = gtk_label_new ("Coords filename:");
  gtk_table_attach_defaults (GTK_TABLE (table), label,
			     0, 1, 3, 4);
  gtk_widget_show (label);

  coords_entry = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (coords_entry),
		      coords_string);
  gtk_widget_show (heads_entry);
  gtk_table_attach_defaults (GTK_TABLE (table), coords_entry,
			     1, 2, 3, 4);
  gtk_widget_show (coords_entry);

  /* notpl fields */
  label = gtk_label_new ("Not plotted mets filename:");
  gtk_table_attach_defaults (GTK_TABLE (table), label,
			     0, 1, 4, 5);
  gtk_widget_show (label);

  notpl_entry = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (notpl_entry),
		      notpl_string);
  gtk_table_attach_defaults (GTK_TABLE (table), notpl_entry,
			     1, 2, 4, 5);
  gtk_widget_show (notpl_entry);

  /* prepare filenames for export button callback */
  heads_filename = g_strdup (gtk_entry_get_text (GTK_ENTRY (heads_entry)));
  coords_filename = g_strdup (gtk_entry_get_text (GTK_ENTRY (coords_entry)));
  notpl_filename = g_strdup (gtk_entry_get_text (GTK_ENTRY (notpl_entry)));

  export_data.heads_filename = heads_filename;
  export_data.coords_filename = coords_filename;
  export_data.notpl_filename = notpl_filename;

  private_export_data = export_data;

  /* buttons */
  hbox = gtk_hbox_new (FALSE, 0);
  gtk_table_attach_defaults (GTK_TABLE (table), hbox,
			     0, 2, 5, 6);
  gtk_widget_show (hbox);

  button = gtk_button_new_with_label ("Export");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (export_form_export_pressed_cb),
		    (gpointer) &private_export_data);
  g_signal_connect_after (G_OBJECT (button), "clicked",
			  G_CALLBACK (export_form_cancel_pressed_cb),
			  export_form_window);
  gtk_box_pack_end (GTK_BOX (hbox), button,
		    FALSE, FALSE, 0);
  gtk_widget_show (button);

  button = gtk_button_new_with_label ("Cancel");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (export_form_cancel_pressed_cb), 
		    export_form_window);
  gtk_box_pack_end (GTK_BOX (hbox), button,
		    FALSE, FALSE, 0);
  gtk_widget_show (button);

  gtk_widget_show (export_form_window);
}
