#include "gui.h"

void create_export_preview_window( corrida_export_data export_data )
{
  /* +--------------------------+
   * |export_preview_window     |
   * | +----------------------+ |
   * | |vpaned1               | |
   * | | +--------------+     | |
   * | | |frame         |     |	|
   * | | |  +---------+ |     |	|
   * | | |  |text_view| |     |	|
   * | | |  +---------+ |     |	|
   * | | +--------------+     |	|
   * | | +-----------------+  |	|
   * | | |vpaned2          |  |	|
   * | | | +-------------+ |  |	|
   * | | | |frame        | |  |	|
   * | | | | +---------+ | |  |	|
   * | | | | |text_view| | |  |	|
   * | | | | +---------+ | |  |	|
   * | | | +-------------+ |  |	|
   * | | | +-------------+ |  |	|
   * | | | |frame        | |  |	|
   * | | | | +---------+ | |  |	|
   * | | | | |text_view| | |  |	|
   * | | | | +---------+ | |  |	|
   * | | | +-------------+ |  |	|
   * | | +-----------------+  |	|
   * | +----------------------+	|
   * +--------------------------+
   */

  GtkWidget *export_preview_window;
  GtkWidget *vpaned1;
  GtkWidget *vpaned2;
  GtkWidget *frame;
  GtkWidget *scrolled_window;
  GtkWidget *text_view;

  GtkTextBuffer *buffer; /* it is not widget ! */
  PangoFontDescription *font_desc;

  font_desc = pango_font_description_from_string ("Courier 12");

  export_preview_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  g_signal_connect (G_OBJECT (export_preview_window), "delete-event",
		    G_CALLBACK (gtk_widget_destroy), NULL);
  g_signal_connect (G_OBJECT (export_preview_window), "destroy-event",
		    G_CALLBACK (gtk_widget_destroy), NULL);
  gtk_window_set_title (GTK_WINDOW (export_preview_window),
			"corrida export preview");
  gtk_window_set_default_size (GTK_WINDOW (export_preview_window),
			       650, 450);

  vpaned1 = gtk_vpaned_new ();
  gtk_paned_set_position (GTK_PANED (vpaned1), 100);
  gtk_container_add (GTK_CONTAINER (export_preview_window),
		     vpaned1);
  gtk_widget_show (vpaned1);

  frame = gtk_frame_new ("Heads");
  gtk_container_set_border_width (GTK_CONTAINER (frame),
                                  GUI_BORDER_SIZE_SMALL);
  gtk_paned_add1 (GTK_PANED (vpaned1), frame);
  gtk_widget_show (frame);

  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_AUTOMATIC,
				  GTK_POLICY_ALWAYS);
  gtk_container_set_border_width (GTK_CONTAINER (scrolled_window),
				  GUI_BORDER_SIZE_SMALL );
  gtk_container_add (GTK_CONTAINER (frame), scrolled_window);
  gtk_widget_show (scrolled_window);

  text_view = gtk_text_view_new ();
  gtk_text_view_set_editable (GTK_TEXT_VIEW (text_view), FALSE);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (text_view), FALSE);
  gtk_widget_modify_font (text_view, font_desc);
  gtk_container_add (GTK_CONTAINER (scrolled_window), text_view);
  gtk_widget_show (text_view);

  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
  gtk_text_buffer_set_text (buffer, export_data.heads->str, -1);

  vpaned2 = gtk_vpaned_new ();
  gtk_paned_set_position (GTK_PANED (vpaned2), 230);
  gtk_paned_add2 (GTK_PANED (vpaned1), vpaned2);
  gtk_widget_show (vpaned2);

  frame = gtk_frame_new ("Plotted meteors");
  gtk_container_set_border_width (GTK_CONTAINER (frame),
                                  GUI_BORDER_SIZE_SMALL);
  gtk_paned_add1 (GTK_PANED (vpaned2), frame);
  gtk_widget_show (frame);  

  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_AUTOMATIC,
				  GTK_POLICY_ALWAYS);
  gtk_container_set_border_width (GTK_CONTAINER (scrolled_window),
				  GUI_BORDER_SIZE_SMALL );
  gtk_container_add (GTK_CONTAINER (frame), scrolled_window);
  gtk_widget_show (scrolled_window);

  text_view = gtk_text_view_new ();
  gtk_text_view_set_editable (GTK_TEXT_VIEW (text_view), FALSE);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (text_view), FALSE);
  gtk_widget_modify_font (text_view, font_desc);
  gtk_container_add (GTK_CONTAINER (scrolled_window), text_view);
  gtk_widget_show (text_view);

  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
  gtk_text_buffer_set_text (buffer, export_data.coords->str, -1);
  
  frame = gtk_frame_new ("Not plotted meteors");
  gtk_container_set_border_width (GTK_CONTAINER (frame),
                                  GUI_BORDER_SIZE_SMALL);
  gtk_paned_add2 (GTK_PANED (vpaned2), frame);
  gtk_widget_show (frame); 

  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_AUTOMATIC,
				  GTK_POLICY_ALWAYS);
  gtk_container_set_border_width (GTK_CONTAINER (scrolled_window),
				  GUI_BORDER_SIZE_SMALL );
  gtk_container_add (GTK_CONTAINER (frame), scrolled_window);
  gtk_widget_show (scrolled_window);

  text_view = gtk_text_view_new ();
  gtk_text_view_set_editable (GTK_TEXT_VIEW (text_view), FALSE);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (text_view), FALSE);
  gtk_widget_modify_font (text_view, font_desc);
  gtk_container_add (GTK_CONTAINER (scrolled_window), text_view);
  gtk_widget_show (text_view);

  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
  gtk_text_buffer_set_text (buffer, export_data.notpl->str, -1);

  pango_font_description_free (font_desc);
  
  gtk_widget_show (export_preview_window);
}
