#include "gui.h"
#include "gui_open_form.h"

void create_open_form_window( void )
{
  GtkWidget *file_selector;
  extern GtkWidget *main_window;

  file_selector = gtk_file_selection_new ("Open file");
  g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
                    "clicked",
                    G_CALLBACK (load_form_ok_button_clicked), (gpointer) file_selector);
  g_signal_connect_after (G_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
			  "clicked",
			  G_CALLBACK (destroy_file_selector), (gpointer) file_selector);
  g_signal_connect_after (G_OBJECT (GTK_FILE_SELECTION (file_selector)->cancel_button),
			  "clicked",
			  G_CALLBACK (destroy_file_selector), (gpointer) file_selector);
  gtk_window_set_modal (GTK_WINDOW (file_selector), TRUE);
  gtk_window_set_transient_for (GTK_WINDOW (file_selector),
                                GTK_WINDOW (main_window));
  
  gtk_widget_show (file_selector);
}

void confirm_open_form_window( void )
{
  GtkWidget *dialog;
  extern GtkWidget *main_window;

  dialog = gtk_message_dialog_new (GTK_WINDOW (main_window),
				   GTK_DIALOG_MODAL
				   | GTK_DIALOG_DESTROY_WITH_PARENT, 
				   GTK_MESSAGE_WARNING,
				   GTK_BUTTONS_OK_CANCEL,
				   "Form was changed!\nAre you sure you want to discard changes?");

  g_signal_connect (G_OBJECT (dialog), "response",
		    G_CALLBACK (open_form_dialog_cb), NULL);
  g_signal_connect_after (G_OBJECT (dialog), "response",
			  G_CALLBACK (gtk_widget_destroy), GTK_OBJECT (dialog));

  gtk_widget_show (dialog);
}

void open_form_dialog_cb( GtkDialog *dialog,
			  gint arg1,
			  gpointer user_data)
{
  if ( arg1 == GTK_RESPONSE_OK ) {
    create_open_form_window ();
  }
}

void load_form_ok_button_clicked( GtkButton *button,
				  gpointer data )
{
  GtkWidget *file_selector;
  gchar filename[MAX_CORRIDA_STRLEN];
  FILE *fp;
  gchar char_buf[MAX_CORRIDA_STRLEN];
  
  GArray *observations_runs_list_local = NULL;
  GArray *meteors_list_local = NULL;
  GArray *charts_meteors_list_local = NULL;

  file_selector = (GtkWidget *)data;
  strcpy (filename,
	  gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selector)));

  fp = fopen (filename, "r");

  if ( fp != NULL ) /* file was succesfully open */ {
    gint it;
    gint read_stat = GUI_OPEN_FORM_READING_STAT_OK;

    gchar observer_code_str[MAX_CORRIDA_STRLEN];
    gchar site_code_str[MAX_CORRIDA_STRLEN];
    gchar date_str[MAX_CORRIDA_STRLEN];
    gchar field_alpha_str[MAX_CORRIDA_STRLEN];
    gchar field_delta_str[MAX_CORRIDA_STRLEN];

    sprintf (char_buf, "file %s opended", filename);
    corrida_printf (char_buf);

    /* read observation */
    if ( read_stat == GUI_OPEN_FORM_READING_STAT_OK && !(feof (fp)) ) {
      fgetl (fp, char_buf, MAX_CORRIDA_STRLEN-1);
      if ( strcmp (char_buf, "#OBSERVATION:") != 0 )
	read_stat = GUI_OPEN_FORM_READING_STAT_ERROR;

      fgetl (fp, observer_code_str, MAX_CORRIDA_STRLEN-1);
      fgetl (fp, site_code_str, MAX_CORRIDA_STRLEN-1);
      fgetl (fp, date_str, MAX_CORRIDA_STRLEN-1);
      fgetl (fp, field_alpha_str, MAX_CORRIDA_STRLEN-1);
      fgetl (fp, field_delta_str, MAX_CORRIDA_STRLEN-1);
    }
    
    /* read observations runs array */
    if ( read_stat == GUI_OPEN_FORM_READING_STAT_OK && !(feof (fp)) ) {
      corrida_observation_run obs_run;

      gchar time_begin[MAX_CORRIDA_STRLEN];
      gchar time_end[MAX_CORRIDA_STRLEN];
      gchar teff[MAX_CORRIDA_STRLEN];
      gchar lm[MAX_CORRIDA_STRLEN];
      gchar f[MAX_CORRIDA_STRLEN];
      gchar last_meteor[MAX_CORRIDA_STRLEN];

      fgetl (fp, char_buf, MAX_CORRIDA_STRLEN-1);
      if ( strcmp (char_buf, "#RUNS:") != 0 )
	read_stat = GUI_OPEN_FORM_READING_STAT_ERROR;

      /* set up a local garray */
      observations_runs_list_local = g_array_new (FALSE, FALSE, 
						  sizeof (corrida_observation_run));

      while ( 1 ) {
	fgetl (fp, char_buf, MAX_CORRIDA_STRLEN-1);

	if ( strcmp (char_buf, "#METEORS:") == 0 || feof (fp) )
	  break;
	else {
	  sscanf (char_buf, "%s %s %s %s %s %s",
		  time_begin, time_end,
		  teff, lm,
		  f, last_meteor);	
	    
	  obs_run.time_begin = g_strdup (time_begin);
	  obs_run.time_end = g_strdup (time_end);
	  obs_run.teff = g_strdup (teff);
	  obs_run.lm = g_strdup (lm);
	  obs_run.f = g_strdup (f);
	  obs_run.last_meteor = g_strdup (last_meteor);
	    
	  g_array_append_val (observations_runs_list_local,
			      obs_run);
	}
      }
    }

    /* read meteors  */
    if ( read_stat == GUI_OPEN_FORM_READING_STAT_OK && !(feof (fp)) ) {
      corrida_meteor_record met_rec;

      gchar no[MAX_CORRIDA_STRLEN];
      gchar time[MAX_CORRIDA_STRLEN];
      gchar mag[MAX_CORRIDA_STRLEN];
      gchar vel[MAX_CORRIDA_STRLEN];
      gchar stream[MAX_CORRIDA_STRLEN];

      if ( strcmp (char_buf, "#METEORS:") != 0 )
	read_stat = GUI_OPEN_FORM_READING_STAT_ERROR;

      /* set up a local garray */
      meteors_list_local = g_array_new (FALSE, FALSE,
					sizeof (corrida_meteor_record));

      while ( 1 ) {
	fgetl (fp, char_buf, MAX_CORRIDA_STRLEN-1);

	if ( strcmp (char_buf, "#CHARTS_METEORS:") == 0 || feof (fp) )
	  break;
	else {
	  sscanf (char_buf, "%s %s %s %s %s",
		  no, time, mag,
		  vel, stream);
	  
	  met_rec.no = g_strdup (no);
	  met_rec.time = g_strdup (time);
	  met_rec.mag = g_strdup (mag);
	  met_rec.vel = g_strdup (vel);
	  met_rec.stream = g_strdup (stream);

	  g_array_append_val (meteors_list_local,
			      met_rec);
	}
      }
    }
    
    /* read charts meteors */
    if ( read_stat == GUI_OPEN_FORM_READING_STAT_OK && !(feof (fp)) ) {
      corrida_charts_meteor_record ch_met_rec;

      gint met_no;
      gint map_no;
      cartesian_coords car_begin;
      cartesian_coords car_end;
      equatorial_coords_degs equ_begin;
      equatorial_coords_degs equ_end;

      if ( strcmp (char_buf, "#CHARTS_METEORS:") != 0 )
	read_stat = GUI_OPEN_FORM_READING_STAT_ERROR;

      /* set up a local garray */
      charts_meteors_list_local = g_array_new (FALSE, FALSE,
					       sizeof (corrida_charts_meteor_record));
      
      while ( 1 ) {
	fgetl (fp, char_buf, MAX_CORRIDA_STRLEN-1);
	
	if ( feof (fp) )
	  break;
	else {
	  sscanf (char_buf, "%d %d %f %f %f %f %f %f %f %f",
		  &met_no, &map_no,
		  &car_begin.x, &car_begin.y,
		  &car_end.x, &car_end.y,
		  &equ_begin.alpha, &equ_begin.delta,
		  &equ_end.alpha, &equ_end.delta);

	  ch_met_rec.met_no = met_no;
	  ch_met_rec.map_no = map_no;

	  ch_met_rec.car_begin.x = car_begin.x;
	  ch_met_rec.car_begin.y = car_begin.y;
	  ch_met_rec.car_end.x = car_end.x;
	  ch_met_rec.car_end.y = car_end.y;

	  ch_met_rec.equ_begin.alpha = equ_begin.alpha;
	  ch_met_rec.equ_begin.delta = equ_begin.delta;
	  ch_met_rec.equ_end.alpha = equ_end.alpha;
	  ch_met_rec.equ_end.delta = equ_end.delta;

	  g_array_append_val (charts_meteors_list_local,
			      ch_met_rec);
	}
      }
    }
    
    if ( read_stat == GUI_OPEN_FORM_READING_STAT_OK ) {
      /* print data to entries and treviews */

      erease_form ();

      /* print data to entries */
      { 
	extern GtkWidget *observer_code_entry;
	extern GtkWidget *site_code_entry;
	extern GtkWidget *date_entry;
	extern GtkWidget *field_alpha_entry;
	extern GtkWidget *field_delta_entry;

	gtk_entry_set_text (GTK_ENTRY (observer_code_entry),
			    observer_code_str);	
	gtk_entry_set_text (GTK_ENTRY (site_code_entry),
			    site_code_str);
	gtk_entry_set_text (GTK_ENTRY (date_entry),
			    date_str);
	gtk_entry_set_text (GTK_ENTRY (field_alpha_entry),
			    field_alpha_str);
	gtk_entry_set_text (GTK_ENTRY (field_delta_entry),
			    field_delta_str);
      }

      /* print data to runs garray and treeview */
      {
	extern GArray *observations_runs_list;	

	/* free global runs garray */
	if ( observations_runs_list != NULL ) 
	  g_array_free (observations_runs_list, TRUE);
	/* point global runs garray on loaded data */
	observations_runs_list = observations_runs_list_local;

	refresh_observations_runs_treeview ();

	print_data_from_runs_list (); /* FIXME: this line may be removed */
      }

      /* print data to meteors garray and treeview */
      {
	extern GArray *meteors_list;

	/* free global meteors garray */
	if ( meteors_list != NULL )
	  g_array_free (meteors_list, TRUE);
	/* point global meteors garray on laded data */
	meteors_list = meteors_list_local;

	refresh_meteors_treeview ();

	print_data_from_meteors_list (); /* FIXME: this line may be removed */
      }

      /* print data to charts meteors garray and treeview */
      {
	extern GArray *charts_meteors_list;

	/* free global runs garray */
	if ( charts_meteors_list != NULL )
	  g_array_free (charts_meteors_list, TRUE);
	/* point global runs garray on loaded data */
	charts_meteors_list = charts_meteors_list_local;

	refresh_charts_meteors_treeview ();
	refresh_chart ();

	print_data_from_charts_meteors_list (); /* FIXME: this line may be removed */
      }

      sprintf (char_buf, "file %s sucessfully loaded", filename);
      corrida_printf (char_buf);
    } else {
      sprintf (char_buf, "ERROR: wrong %s file format", filename);
      corrida_printf (char_buf);
    }

    fclose (fp);
  } else { /* file was not open */
    sprintf (char_buf, "ERROR: unable to open %s file for reading", filename);
    corrida_printf (char_buf);
  }
}
