#include "gui.h"
#include "gui_save_form.h"

void create_save_form_window( void )
{
  GtkWidget *file_selector;
  extern GtkWidget *main_window;

  file_selector = gtk_file_selection_new ("Save form as");
  g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
		    "clicked",
		    G_CALLBACK (save_form_ok_button_clicked), (gpointer) file_selector);

  g_signal_connect_after (G_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
		    "clicked",
		    G_CALLBACK (destroy_file_selector), (gpointer) file_selector);
  g_signal_connect_after (G_OBJECT (GTK_FILE_SELECTION (file_selector)->cancel_button),
		    "clicked",
		    G_CALLBACK (destroy_file_selector), (gpointer) file_selector);
  gtk_window_set_modal (GTK_WINDOW (file_selector), TRUE);
  gtk_window_set_transient_for (GTK_WINDOW (file_selector),
				GTK_WINDOW (main_window));

  gtk_widget_show (file_selector);
}

void save_form_ok_button_clicked( GtkButton *button,
				  gpointer data )
{
  GtkWidget *file_selector;
  gchar filename[MAX_CORRIDA_STRLEN];
  FILE *fp;
  gchar char_buf[MAX_CORRIDA_STRLEN];

  file_selector = ((GtkWidget *)data);

  strcpy (filename,
	  gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selector)));
  fp = fopen (filename, "w");

  if ( fp != NULL ) { /* file succesfully open, printf data to file */
    gint it;

    { /* print observation */
      extern GtkWidget *observer_code_entry;
      extern GtkWidget *site_code_entry;
      extern GtkWidget *date_entry;
      extern GtkWidget *field_alpha_entry;
      extern GtkWidget *field_delta_entry;
      
      fprintf (fp, "#OBSERVATION:\n");
      
      strcpy (char_buf,
	      gtk_entry_get_text (GTK_ENTRY (observer_code_entry)));
      fprintf (fp, "%s\n", char_buf);
      
      strcpy (char_buf,
	      gtk_entry_get_text (GTK_ENTRY (site_code_entry)));
      fprintf (fp, "%s\n", char_buf);

      strcpy (char_buf,
	      gtk_entry_get_text (GTK_ENTRY (date_entry)));
      fprintf (fp, "%s\n", char_buf);

      strcpy (char_buf,
	      gtk_entry_get_text (GTK_ENTRY (field_alpha_entry)));
      fprintf (fp, "%s\n", char_buf);

      strcpy (char_buf,
	      gtk_entry_get_text (GTK_ENTRY (field_delta_entry)));
      fprintf (fp, "%s\n", char_buf);
    }
  
    { /* print observation runs garray */   
      extern GArray *observations_runs_list;
      corrida_observation_run run;
      
      fprintf (fp, "#RUNS:\n");
      
      for ( it = 0; it < observations_runs_list->len; it++ ) {
	run = g_array_index (observations_runs_list,
			     corrida_observation_run,
			     it);
	fprintf (fp, "%s %s %s %s %s %s\n",
		 run.time_begin, run.time_end, run.teff,
		 run.lm, run.f, run.last_meteor);
      }
    }

    { /* print meteors garray */
      extern GArray *meteors_list;
      corrida_meteor_record met_rec;

      fprintf (fp, "#METEORS:\n");
      
      for ( it = 0; it < meteors_list->len; it++ ) {
	met_rec = g_array_index (meteors_list,
				  corrida_meteor_record,
				  it);
	fprintf (fp, "%s %s %s %s %s\n",
		 met_rec.no,
		 met_rec.time,
		 met_rec.mag,
		 met_rec.vel,
		 met_rec.stream);
      }
    }
     
    { /* print chart meteors garray */
      extern GArray *charts_meteors_list;
      corrida_charts_meteor_record ch_met_rec;

      fprintf (fp, "#CHARTS_METEORS:\n");
      
      for ( it = 0; it < charts_meteors_list->len; it++ ) {
	ch_met_rec = g_array_index (charts_meteors_list,
				    corrida_charts_meteor_record,
				    it);
	fprintf (fp, "%d %d",
		 ch_met_rec.met_no, ch_met_rec.map_no);
	fprintf (fp, " %f %f",
		 ch_met_rec.car_begin.x, ch_met_rec.car_begin.y);
	fprintf (fp, " %f %f",
		 ch_met_rec.car_end.x, ch_met_rec.car_end.y);
	fprintf (fp, " %f %f",
		 ch_met_rec.equ_begin.alpha, ch_met_rec.equ_begin.delta);
	fprintf (fp, " %f %f",
		 ch_met_rec.equ_end.alpha, ch_met_rec.equ_end.delta);
	fprintf (fp, "\n");
      }
    }

    sprintf (char_buf, "file %s saved", filename);
    corrida_printf (char_buf);

    internal_data_set_edition_stat (INTERNAL_DATA_NOT_EDITED);

    fclose (fp);
  } else { /* file was not open */
    sprintf (char_buf, "ERROR: unable to open %s file for writing", filename);
    corrida_printf (char_buf);
  }
}
