#include "misc.h"

/* corrida_init: init corrida services */
void corrida_init( int  *argc,
		 char ***argv )
{
  char string[MAX_CORRIDA_STRLEN];

  sprintf (string, "ver. no. %.2f", CORRIDA_VER_NO);
  corrida_printf (string);

  corrida_printf ("initializing corrida services...");

  gnomonic_init ();
  internal_data_init ();
  forms_init (argc, argv);
  corrida_gui_init (argc, argv);

  corrida_printf ("services initialized");

  corrida_gui_start ();
}


/* corrida_deg2rad: convert degrees into radians */
float corrida_deg2rad( float deg )
{
  return (deg * G_PI / 180.0);
}

/* corrida_rad2deg: convert radians into degrees */
float corrida_rad2deg( float rad )
{
  return (rad * 180.0 / G_PI);
}

/* equ_degs2equ_hours: convert equatorial coordinates
 *                     in degrees into coordinate in hours
 */
int equ_degs2equ_hours( equatorial_coords_degs eqcoords,
			equatorial_coords_hours *eqcoords_h )
{
  int a_hour, a_min;
  float a_sec;
  
  int d_deg;
  float d_min;

  float alpha = eqcoords.alpha;
  float delta = eqcoords.delta;

  float buf_float1, buf_float2;
  double buf_double;

  buf_float1 = alpha / 15;
  buf_float2 = modf (buf_float1, &buf_double);
  a_hour = (int) buf_double;
  
  buf_float1 = buf_float2 * 60.0;
  buf_float2 = modf (buf_float1, &buf_double);
  a_min = (int) buf_double;
  
  a_sec = buf_float2 * 60.0;

  buf_float2 = modf (delta, &buf_double);
  d_deg = (int) buf_double;
  
  d_min = buf_float2 * 60.0;

  eqcoords_h->alpha_hour = a_hour;
  eqcoords_h->alpha_min  = a_min;
  eqcoords_h->alpha_sec  = a_sec;

  eqcoords_h->delta_deg = d_deg;
  eqcoords_h->delta_min = d_min;
  
  return 1;
}
/* equ_degs2equ_degs_string: convert equatorial_coords_degs 
 *                           into string in degs
 */
char *equ_degs2equ_degs_string( equatorial_coords_degs eqcoords_d,
				char *string )
{
  float alpha = eqcoords_d.alpha;
  float delta = eqcoords_d.delta;

  sprintf (string, "alpha: %6.2f deg\ndelta: %+6.2f deg",
	   alpha, delta);

  return string;
}

/* equ_degs2equ_hours_string: convert equatorial_coords_degs
 *                            into string in hours
 */
char *equ_degs2equ_hours_string( equatorial_coords_degs eqcoords_d,
				 char *string )
{
  equatorial_coords_hours eqcoords_h;

  equ_degs2equ_hours (eqcoords_d, &eqcoords_h);
  equ_hours2equ_hours_string (eqcoords_h, string);

  return string;
}

/* equ_hours2equ_hours_string: convert equatorial_coords_hours
 *                             into string in hours
 */
char *equ_hours2equ_hours_string( equatorial_coords_hours eqcoords_h,
				  char *string)
{
  char sign;

  int a_hour  = eqcoords_h.alpha_hour;
  int a_min   = eqcoords_h.alpha_min;
  float a_sec = eqcoords_h.alpha_sec;
  
  int d_deg   = eqcoords_h.delta_deg;
  float d_min = eqcoords_h.delta_min;

  if (a_sec >= 30)
    a_min++;
  if (a_min == 60) {
    a_min = 0;
    a_hour++;
  }
  if (a_hour == 24)
    a_hour = 0;

  if (d_deg < 0 || d_min < 0)
    sign = '-';
  else
    sign = '+';

  d_deg = (int) fabs (d_deg);
  d_min = fabs (d_min);

  if (d_min == 60) {
    d_min = 60;
    d_deg++;
  }

  sprintf (string, "alpha:  %2dh %2dm\ndelta: %c%2dd %2.0fm",
	   a_hour, a_min, sign, d_deg, d_min);

  return string;
}

int fgetl( FILE *file, gchar *line, int max_strlen )
{
  gchar c;
  gint line_len = 0;

  sprintf (line, "\0");
  
  while ( !(feof (file)) 
	  && (c = fgetc (file)) != '\n' 
	  && line_len < max_strlen ) {
    line[line_len] = c;
    line_len++;
  }
  
  line[line_len] = '\0';

  return line_len;
}

gdouble compute_angle( gdouble x, gdouble y )
{
  if ( x < 0.02 && x > - 0.02 ) /* 90 or 270 degs */
    {
      if ( y > 0 ) 
	return G_PI / 2.0;
      else 
	return 3.0 * G_PI / 2.0;
    } 
  else if ( y < 0.02 && y > - 0.02 ) /* 0 or 180 degs */
    {
      if ( x > 0 )
	return 0;
      else
	return G_PI;
    }
  else
    {
      gdouble quarter;

      if ( x > 0 && y > 0 ) /* first quarter */
	quarter = 0;
      else if ( x < 0 && y > 0 ) { /* second quarter */
	gdouble buf;
	
	buf = x;
	x = y;
	y = buf;

	quarter = G_PI / 2.0;
      }
      else if ( x < 0 && y < 0 ) /* third quarter */
	quarter = G_PI;
      else if ( x > 0 && y < 0 ) { /* fourth quarter */
	gdouble buf;
	
	buf = x;
	x = y;
	y = buf;

	quarter = 3.0 * G_PI / 2.0;
      }

      if ( x < 0 )
	x = -x;

      if ( y < 0 )
	y = -y;
      
      return atan ( y / x ) + quarter;
    }
}
void string_to_upper_case (char string[])
{
  while ( *string != '\0' )
    {
      *string = toupper (*string);
      string++;
    }
}
